@@ -2,6 +2,10 @@ class AssignmentPolicyError(Exception):
     pass
 
 
+class CreationPolicyError(Exception):
+    pass
+
+
 class IllegalTaskSubmission(Exception):
     pass
 
@@ -18,11 +22,11 @@ class ModelSaveError(Exception):
     pass
 
 
-class NoTaskAvailable(Exception):
+class ReviewPolicyError(Exception):
     pass
 
 
-class ReviewPolicyError(Exception):
+class SanityBotError(Exception):
     pass
 
 
@@ -60,3 +64,11 @@ class WorkflowError(Exception):
 
 class TimerError(Exception):
     pass
+
+
+class ProjectStatusError(Exception):
+    pass
+
+
+class TodoListTemplateValidationError(Exception):
+    pass
