@@ -1,26 +1,30 @@
 import json
+from unittest.mock import patch
 
 from django.conf import settings
 from django.contrib.auth.models import Group
-from django.core.urlresolvers import reverse
+from django.urls import reverse
 from django.test import Client
+from django.test import override_settings
 
 from orchestra.core.errors import InvalidRevertError
+from orchestra.models import Project
 from orchestra.models import Task
 from orchestra.models import TaskAssignment
-from orchestra.models import Project
 from orchestra.project_api.serializers import IterationSerializer
-from orchestra.project_api.serializers import TaskSerializer
 from orchestra.project_api.serializers import TaskAssignmentSerializer
+from orchestra.project_api.serializers import TaskSerializer
 from orchestra.tests.helpers import OrchestraTestCase
-from orchestra.tests.helpers.fixtures import setup_complete_task
-from orchestra.tests.helpers.fixtures import setup_models
 from orchestra.tests.helpers.fixtures import UserFactory
 from orchestra.tests.helpers.fixtures import WorkerFactory
+from orchestra.tests.helpers.fixtures import setup_complete_task
+from orchestra.tests.helpers.fixtures import setup_models
 from orchestra.tests.helpers.iterations import verify_iterations
 from orchestra.utils.load_json import load_encoded_json
 from orchestra.utils.revert import RevertChange
 from orchestra.utils.task_lifecycle import create_subsequent_tasks
+from orchestra.utils.task_lifecycle import assign_task
+from orchestra.utils.task_lifecycle import complete_and_skip_task
 from orchestra.utils.task_properties import assignment_history
 from orchestra.utils.task_properties import current_assignment
 from orchestra.utils.task_properties import get_iteration_history
@@ -37,7 +41,7 @@ class ProjectManagementAPITestCase(OrchestraTestCase):
                             first_name='project_management',
                             last_name='project_management',
                             password='project_management',
-                            email='project_management@unlimitedlabs.com'))
+                            email='project_management@b12.io'))
         self.worker = WorkerFactory(user=user)
         user.groups.add(self.project_admin_group)
 
@@ -55,14 +59,14 @@ class ProjectManagementAPITestCase(OrchestraTestCase):
                 'project_id': project.id,
             }),
             content_type='application/json')
-        self.assertEquals(response.status_code, 200)
+        self.assertEqual(response.status_code, 200)
         returned = load_encoded_json(response.content)
 
         # Skipping the `tasks` key/value pair for this test
         expected_project = {
             'task_class': project.task_class,
             'start_datetime': '2015-10-12T00:00:00Z',
-            'team_messages_url': None,
+            'scratchpad_url': None,
             'admin_url': settings.ORCHESTRA_URL + reverse(
                 'admin:orchestra_project_change',
                 args=(project.id,)),
@@ -72,11 +76,12 @@ class ProjectManagementAPITestCase(OrchestraTestCase):
             'id': project.id,
             'workflow_slug': project.workflow_version.workflow.slug
         }
-        self.assertEquals(
+        returned_project = returned[str(project.id)]['project']
+        self.assertEqual(
             expected_project,
-            {k: returned['project'][k] for k in expected_project.keys()})
+            {k: returned_project[k] for k in expected_project.keys()})
 
-        sample_task = returned['tasks']['step1']
+        sample_task = returned[str(project.id)]['tasks']['step1']
         task = project.tasks.get(step__slug='step1')
         expected_task = {
             'status': 'Post-review Processing',
@@ -92,7 +97,7 @@ class ProjectManagementAPITestCase(OrchestraTestCase):
             'id': task.id
         }
 
-        self.assertEquals(
+        self.assertEqual(
             expected_task,
             {k: sample_task[k] for k in expected_task.keys()})
 
@@ -110,6 +115,8 @@ class ProjectManagementAPITestCase(OrchestraTestCase):
                 'first_name': assignment.worker.user.first_name,
                 'last_name': assignment.worker.user.last_name,
                 'id': assignment.worker.id,
+                'slack_username': assignment.worker.slack_username,
+                'slack_user_id': assignment.worker.slack_user_id
             },
             'iterations': [
                 {
@@ -128,7 +135,7 @@ class ProjectManagementAPITestCase(OrchestraTestCase):
             'id': assignment.id
         }
 
-        self.assertEquals(
+        self.assertEqual(
             expected_assignment,
             {k: sample_assignment[k] for k in expected_assignment.keys()})
 
@@ -144,9 +151,9 @@ class ProjectManagementAPITestCase(OrchestraTestCase):
                 'assignment_id': entry_assignment.id
             }),
             content_type='application/json')
-        self.assertEquals(response.status_code, 200)
+        self.assertEqual(response.status_code, 200)
         entry_assignment.refresh_from_db()
-        self.assertEquals(entry_assignment.worker, self.workers[0])
+        self.assertEqual(entry_assignment.worker, self.workers[0])
 
         # Reassign entry-level assignment to reviewer
         entry_assignment.refresh_from_db()
@@ -158,9 +165,9 @@ class ProjectManagementAPITestCase(OrchestraTestCase):
                 'assignment_id': entry_assignment.id
             }),
             content_type='application/json')
-        self.assertEquals(response.status_code, 200)
+        self.assertEqual(response.status_code, 200)
         entry_assignment.refresh_from_db()
-        self.assertEquals(entry_assignment.worker, self.workers[1])
+        self.assertEqual(entry_assignment.worker, self.workers[1])
 
         # Attempt to reassign assignment to worker already working on task
         first_review_assignment = task.assignments.get(assignment_counter=1)
@@ -172,12 +179,12 @@ class ProjectManagementAPITestCase(OrchestraTestCase):
                 'assignment_id': first_review_assignment.id
             }),
             content_type='application/json')
-        self.assertEquals(response.status_code, 400)
+        self.assertEqual(response.status_code, 400)
         first_review_assignment.refresh_from_db()
         self.assertTrue(first_review_assignment.worker, self.workers[1])
         returned = load_encoded_json(response.content)
-        self.assertEquals(returned['message'],
-                          'Worker already assigned to this task.')
+        self.assertEqual(returned['message'],
+                         'Worker already assigned to this task.')
 
         # Attempt to reassign review assignment to entry-level worker
         first_review_assignment = task.assignments.get(assignment_counter=1)
@@ -189,12 +196,12 @@ class ProjectManagementAPITestCase(OrchestraTestCase):
                 'assignment_id': first_review_assignment.id
             }),
             content_type='application/json')
-        self.assertEquals(response.status_code, 400)
+        self.assertEqual(response.status_code, 400)
         first_review_assignment.refresh_from_db()
         self.assertTrue(first_review_assignment.worker, self.workers[0])
         returned = load_encoded_json(response.content)
-        self.assertEquals(returned['message'],
-                          'Worker not certified for this assignment.')
+        self.assertEqual(returned['message'],
+                         'Worker not certified for this assignment.')
 
         # Reassign review assignment to another reviewer
         second_review_assignment = task.assignments.get(assignment_counter=2)
@@ -206,11 +213,13 @@ class ProjectManagementAPITestCase(OrchestraTestCase):
                 'assignment_id': second_review_assignment.id
             }),
             content_type='application/json')
-        self.assertEquals(response.status_code, 200)
+        self.assertEqual(response.status_code, 200)
         second_review_assignment.refresh_from_db()
-        self.assertEquals(second_review_assignment.worker, self.workers[3])
+        self.assertEqual(second_review_assignment.worker, self.workers[3])
 
     def test_complete_and_skip_task_api(self):
+        # This task, when submitted, will be moved into reviewing
+        # state because it's already been assigned.
         task = self.tasks['project_management_task']
         response = self.api_client.post(
             reverse(
@@ -220,18 +229,58 @@ class ProjectManagementAPITestCase(OrchestraTestCase):
                 'task_id': task.id,
             }),
             content_type='application/json')
-        self.assertEquals(response.status_code, 200)
+        self.assertEqual(response.status_code, 200)
         task.refresh_from_db()
-        self.assertEquals(task.status, Task.Status.COMPLETE)
-        for assignment in task.assignments.all():
-            self.assertEquals(
-                assignment.status, TaskAssignment.Status.SUBMITTED)
-
+        self.assertEqual(task.status, Task.Status.REVIEWING)
+        assignment_statuses = [
+            TaskAssignment.Status.SUBMITTED, TaskAssignment.Status.SUBMITTED,
+            TaskAssignment.Status.PROCESSING]
+        for index, assignment in enumerate(
+                task.assignments.all().order_by('assignment_counter')):
+            self.assertEqual(
+                assignment.status, assignment_statuses[index])
         # Check that dependent tasks have already been created
-        # TODO(jrbotros): Create a `get_dependent_tasks` function
         num_tasks = task.project.tasks.count()
         create_subsequent_tasks(task.project)
-        self.assertEquals(task.project.tasks.count(), num_tasks)
+        self.assertEqual(task.project.tasks.count(), num_tasks)
+
+        # This task, when submitted, will be moved into a completed
+        # state because it has no assignments.
+        task = self.tasks['awaiting_processing']
+        response = self.api_client.post(
+            reverse(
+                'orchestra:orchestra:project_management:'
+                'complete_and_skip_task'),
+            json.dumps({
+                'task_id': task.id,
+            }),
+            content_type='application/json')
+        self.assertEqual(response.status_code, 200)
+        task.refresh_from_db()
+        self.assertEqual(task.status, Task.Status.COMPLETE)
+
+    @override_settings(ORCHESTRA_SLACK_EXPERTS_ENABLED=True)
+    @patch(
+        'orchestra.communication.tests.helpers.slack.Conversations.unarchive')
+    @patch('orchestra.communication.tests.helpers.slack.Conversations.archive')
+    def test_unarchive_slack_channel_api(self, mock_archive, mock_unarchive):
+        project = self.projects['single_human_step']
+        create_subsequent_tasks(project)
+        task = Task.objects.get(project=project)
+        assign_task(self.workers[1].id, task.id)
+        complete_and_skip_task(task.id)
+        create_subsequent_tasks(project)
+        self.assertTrue(mock_archive.called)
+
+        response = self.api_client.post(
+            reverse(('orchestra:orchestra:project_management'
+                     ':unarchive_slack_channel')),
+            json.dumps({
+                'project_id': project.id,
+            }),
+            content_type='application/json')
+        self.assertEqual(response.status_code, 200)
+        self.assertTrue(mock_unarchive.called)
 
     def test_end_project_api(self):
         project = self.projects['project_to_end']
@@ -241,11 +290,42 @@ class ProjectManagementAPITestCase(OrchestraTestCase):
                 'project_id': project.id,
             }),
             content_type='application/json')
-        self.assertEquals(response.status_code, 200)
+        self.assertEqual(response.status_code, 200)
         project.refresh_from_db()
-        self.assertEquals(project.status, Project.Status.ABORTED)
+        self.assertEqual(project.status, Project.Status.ABORTED)
         for task in Task.objects.filter(project=project):
-            self.assertEquals(task.status, Task.Status.ABORTED)
+            self.assertEqual(task.status, Task.Status.ABORTED)
+
+    def test_set_project_status_api(self):
+        project = self.projects['project_to_set_status']
+        status_choices = dict(Project.STATUS_CHOICES)
+        paused_status = status_choices[Project.Status.PAUSED]
+        active_status = status_choices[Project.Status.ACTIVE]
+        response = self.api_client.post(
+            reverse((
+                'orchestra:orchestra:project_management:'
+                'set_project_status')),
+            json.dumps({
+                'project_id': project.id,
+                'status': paused_status
+            }),
+            content_type='application/json')
+        self.assertEqual(response.status_code, 200)
+        project.refresh_from_db()
+        self.assertEqual(project.status, Project.Status.PAUSED)
+
+        response = self.api_client.post(
+            reverse((
+                'orchestra:orchestra:project_management:'
+                'set_project_status')),
+            json.dumps({
+                'project_id': project.id,
+                'status': active_status
+            }),
+            content_type='application/json')
+        self.assertEqual(response.status_code, 200)
+        project.refresh_from_db()
+        self.assertEqual(project.status, Project.Status.ACTIVE)
 
     def test_invalid_revert_aborted_task(self):
         task = setup_complete_task(self)
@@ -440,15 +520,15 @@ class ProjectManagementAPITestCase(OrchestraTestCase):
                             revert_before=False):
         response = self._revert_task(
             task, iteration, revert_before=revert_before, commit=False)
-        self.assertEquals(response.status_code, 200)
+        self.assertEqual(response.status_code, 200)
         task.refresh_from_db()
         fake_audit = load_encoded_json(response.content)
         self.assertEqual(fake_audit, expected_audit)
 
-        self.assertEquals(task.status, Task.Status.COMPLETE)
-        self.assertEquals(task.assignments.count(), 2)
+        self.assertEqual(task.status, Task.Status.COMPLETE)
+        self.assertEqual(task.assignments.count(), 2)
         for assignment in task.assignments.all():
-            self.assertEquals(assignment.iterations.count(), 2)
+            self.assertEqual(assignment.iterations.count(), 2)
 
         response = self._revert_task(
             task, iteration, revert_before=revert_before, commit=True)
@@ -464,7 +544,7 @@ class ProjectManagementAPITestCase(OrchestraTestCase):
         verify_iterations(task.id)
 
         if num_iterations:
-            self.assertEquals(
+            self.assertEqual(
                 current_assignment(task).in_progress_task_data,
                 latest_data)
 
@@ -524,3 +604,34 @@ class ProjectManagementAPITestCase(OrchestraTestCase):
                     audit['assignments'][i][
                         'iterations'][j]['change'] = iteration_change
         return audit
+
+    @patch('orchestra.interface_api.project_management.views.StaffBot.restaff')
+    @patch('orchestra.interface_api.project_management.views.StaffBot.staff')
+    def test_staff_task(self, mock_staff, mock_restaff):
+        # Staff a task with no assignment
+        unassigned_task = self.tasks['awaiting_processing']
+        response = self.api_client.post(
+            reverse(
+                'orchestra:orchestra:project_management:staff_task'),
+            json.dumps({
+                'task_id': unassigned_task.id
+            }),
+            content_type='application/json')
+        self.assertTrue(mock_staff.called)
+        self.assertEqual(response.status_code, 200)
+        is_restaff = load_encoded_json(response.content)['is_restaff']
+        self.assertFalse(is_restaff)
+
+        # Restaff a task with an assignment
+        assigned_task = self.tasks['review_task']
+        response2 = self.api_client.post(
+            reverse(
+                'orchestra:orchestra:project_management:staff_task'),
+            json.dumps({
+                'task_id': assigned_task.id
+            }),
+            content_type='application/json')
+        self.assertTrue(mock_restaff.called)
+        self.assertEqual(response2.status_code, 200)
+        is_restaff2 = load_encoded_json(response2.content)['is_restaff']
+        self.assertTrue(is_restaff2)
