@@ -1,8 +1,9 @@
 # -*- coding: utf-8 -*-
 from __future__ import unicode_literals
 
-from django.db import models, migrations
 import django.utils.timezone
+from django.db import migrations
+from django.db import models
 
 
 class Migration(migrations.Migration):
