@@ -1,13 +1,15 @@
 from datetime import timedelta
-from dateutil.parser import parse
 from unittest.mock import patch
 
 import factory
+from dateutil.parser import parse
 from django.contrib.auth.models import User
 from django.test import Client
 from django.test import override_settings
 from django.utils import timezone
 
+from orchestra.communication.slack import create_project_slack_group
+from orchestra.core.errors import WorkflowError
 from orchestra.models import CommunicationPreference
 from orchestra.models import Iteration
 from orchestra.models import StaffBotRequest
@@ -15,25 +17,106 @@ from orchestra.models import StaffingRequestInquiry
 from orchestra.models import StaffingResponse
 from orchestra.models import Step
 from orchestra.models import Task
+from orchestra.models import Todo
 from orchestra.models import TaskAssignment
+from orchestra.models import TodoListTemplate
 from orchestra.models import WorkerCertification
 from orchestra.models import Workflow
 from orchestra.models import WorkflowVersion
-from orchestra.communication.slack import create_project_slack_group
+from orchestra.tests.helpers.iterations import verify_iterations
+from orchestra.tests.helpers.workflow import workflow_fixtures
+from orchestra.utils.datetime_utils import first_day_of_the_week
 from orchestra.utils.task_lifecycle import assign_task
 from orchestra.utils.task_lifecycle import submit_task
 from orchestra.utils.task_properties import assignment_history
 from orchestra.utils.task_properties import current_assignment
 from orchestra.utils.task_properties import get_iteration_history
-from orchestra.tests.helpers.iterations import verify_iterations
-from orchestra.tests.helpers.workflow import workflow_fixtures
 from orchestra.workflow.defaults import get_default_assignment_policy
+from orchestra.workflow.defaults import get_default_creation_policy
 from orchestra.workflow.defaults import get_default_review_policy
 
 BASE_DATETIME = parse('2015-10-12T00:00:00+00:00')
 ITERATION_DURATION = timedelta(hours=1)
 PICKUP_DELAY = timedelta(hours=1)
 
+TODO_TEMPLATE_GOOD_CSV_TEXT = """Slug,Remove if,Skip if
+,[],[],the root
+todo-parent-slug,[],[],,todo parent 1
+,[],"[{""prop"": {""value"": true, ""operator"": ""==""}}]",,,todo child 1-1
+,"[{""prop"": {""value"": true, ""operator"": ""==""}}]",[],,todo parent 2
+,[],[],,,todo child 2-1
+,[],[],,,,todo child 2-1-1
+,[],[],,,todo child 2-2
+"""
+
+TODO_TEMPLATE_BAD_HEADER_CSV_TEXT = """,Remove if,Skip if
+[],[],the root
+"""
+
+TODO_TEMPLATE_TWO_ENTRIES_CSV_TEXT = """Slug,Remove if,Skip if
+,[],[],the root,the second root
+"""
+
+TODO_TEMPLATE_INVALID_PARENT_CSV_TEXT = """Slug,Remove if,Skip if
+,[],[],the root
+,[],[],,,,,,an impossible child
+"""
+
+TODO_TEMPLATE_NESTED_TODOS = {
+    'description': 'the root',
+    'slug': None,
+    'id': 0,
+    'items': [
+        {
+            'id': 2,
+            'description': 'todo parent 2',
+            'slug': None,
+            'items': [
+                {
+                    'id': 22,
+                    'description': 'todo child 2-2',
+                    'slug': None,
+                    'items': []
+                },
+                {
+                    'id': 21,
+                    'description': 'todo child 2-1',
+                    'slug': None,
+                    'items': [
+                        {
+                            'id': 211,
+                            'description': 'todo child 2-1-1',
+                            'slug': None,
+                            'items': []
+                        }
+                    ]
+                }
+            ],
+            'remove_if': [{
+                'prop': {
+                    'operator': '==',
+                    'value': True
+                }
+            }]
+        },
+        {
+            'id': 1,
+            'description': 'todo parent 1',
+            'slug': 'todo-parent-slug',
+            'items': [{
+                'id': 11,
+                'description': 'todo child 1-1',
+                'slug': None,
+                'items': [],
+                'skip_if': [{
+                    'prop': {
+                        'operator': '==',
+                        'value': True
+                    }
+                }]
+            }]
+        }]}
+
 
 def get_detailed_description(task_details, text=None):
     if text is None:
@@ -87,6 +170,13 @@ class WorkerFactory(factory.django.DjangoModelFactory):
         model = 'orchestra.Worker'
 
 
+class WorkerAvailabilityFactory(factory.django.DjangoModelFactory):
+    week = factory.LazyFunction(first_day_of_the_week)
+
+    class Meta:
+        model = 'orchestra.WorkerAvailability'
+
+
 class CertificationFactory(factory.django.DjangoModelFactory):
 
     slug = factory.Sequence(
@@ -106,6 +196,7 @@ class WorkerCertificationFactory(factory.django.DjangoModelFactory):
     worker = factory.SubFactory(WorkerFactory)
     task_class = WorkerCertification.TaskClass.REAL
     role = WorkerCertification.Role.ENTRY_LEVEL
+    staffing_priority = 0
 
     class Meta:
         model = 'orchestra.WorkerCertification'
@@ -155,6 +246,43 @@ class TimeEntryFactory(factory.django.DjangoModelFactory):
         model = 'orchestra.TimeEntry'
 
 
+class TodoFactory(factory.django.DjangoModelFactory):
+    task = factory.SubFactory(TaskFactory)
+    status = Todo.Status.PENDING.value
+    title = factory.Sequence(
+        lambda n: 'Title {}'.format(n))
+    start_by_datetime = None
+    due_datetime = None
+    slug = None
+
+    class Meta:
+        model = 'orchestra.Todo'
+
+
+class TodoQAFactory(factory.django.DjangoModelFactory):
+    todo = factory.SubFactory(TodoFactory)
+    approved = True
+    comment = factory.Sequence(
+        lambda n: 'Comment {}'.format(n))
+
+    class Meta:
+        model = 'orchestra.TodoQA'
+
+
+class TodoListTemplateFactory(factory.django.DjangoModelFactory):
+    slug = factory.Sequence(
+        lambda n: 'todolist_template_{}'.format(n))
+    name = factory.Sequence(
+        lambda n: 'TodoList template {}'.format(n))
+    description = factory.Sequence(
+        lambda n: 'Description {}'.format(n))
+    creator = None
+    todos = {'items': []}
+
+    class Meta:
+        model = 'orchestra.TodoListTemplate'
+
+
 class CommunicationPreferenceFactory(factory.django.DjangoModelFactory):
 
     worker = factory.SubFactory(WorkerFactory)
@@ -271,9 +399,14 @@ def setup_models(test_case):
         'reject_rev_proj': 'test_workflow',
         'aborted_project': 'test_workflow',
         'project_to_end': 'test_workflow',
+        'project_to_set_status': 'test_workflow',
         'assignment_policy': 'assignment_policy_workflow',
         'project_management_project': 'test_workflow',
         'staffbot_assignment_policy': 'staffbot_assignment_policy_workflow',
+        'creation_policy': 'creation_policy_workflow',
+        'next_todo_proj': 'test_workflow',
+        'sanitybot': 'sanitybot_workflow',
+        'single_human_step': 'single_step_no_review',
     }
 
     # Task generation data
@@ -326,15 +459,43 @@ def setup_models(test_case):
                 (4, test_data, TaskAssignment.Status.PROCESSING)
             ]
         },
+        'next_todo_task': {
+            'project_name': 'next_todo_proj',
+            'status': Task.Status.PROCESSING,
+            'assignments': [
+                (5, {}, TaskAssignment.Status.PROCESSING)
+            ]
+        },
+    }
+
+    todolist_template_slug = 'project-checklist'
+
+    todolist_templates = {
+        todolist_template_slug: {
+            'name': 'Project checklist',
+            'slug': todolist_template_slug
+        }
     }
 
     # Create the objects
+    _setup_todolist_templates(test_case, todolist_templates)
     _setup_workflows(test_case, workflows)
     _setup_workers(test_case, workers)
     _setup_projects(test_case, projects)
     _setup_tasks(test_case, tasks)
 
 
+def _setup_todolist_templates(test_case, templates):
+    test_case.todolist_templates = {}
+    for template_idx, template_key in enumerate(templates):
+        template_details = templates[template_key]
+        template = TodoListTemplateFactory(
+            slug=template_details['slug'],
+            name=template_details['name'],
+        )
+        test_case.todolist_templates[template_details['slug']] = template
+
+
 def _setup_workflows(test_case, workflows):
     # Create workflows
     test_case.workflows = {}
@@ -371,6 +532,7 @@ def _setup_workflows(test_case, workflows):
                 slug=version_details['slug'],
                 name=version_details['name'],
                 description=version_details['description'],
+                sanity_checks=version_details.get('sanity_checks', {})
             )
             test_case.workflow_versions[version_details['slug']] = version
 
@@ -398,10 +560,16 @@ def _setup_workflows(test_case, workflows):
                         'assignment_policy',
                         get_default_assignment_policy(is_human)
                     ),
+                    creation_policy=step_details.get(
+                        'creation_policy',
+                        get_default_creation_policy()),
                     user_interface=step_details.get('user_interface', {}),
                     execution_function=step_details.get('execution_function',
                                                         {}),
                 )
+                _set_step_relations(step, step_details,
+                                    'todolist_templates_to_apply',
+                                    TodoListTemplate)
                 workflow_steps[step.slug] = step
 
                 # Add required certifications
@@ -443,6 +611,18 @@ def _add_step_dependencies(step_dict, existing_workflow_steps, attr):
     return backrefs
 
 
+def _set_step_relations(step, step_data, relation_attr, relation_model,
+                        **model_filters):
+    relation_slugs = set(step_data.get(relation_attr, []))
+    relations = list(relation_model.objects.filter(
+        slug__in=relation_slugs, **model_filters))
+    if len(relations) != len(relation_slugs):
+        raise WorkflowError(
+                            '{}.{} contains a non-existent slug.'
+                            .format(step_data['slug'], relation_attr))
+    getattr(step, relation_attr).set(relations)
+
+
 def _create_backrefs(workflow_steps, backrefs):
     for backref in backrefs:
         dependency = workflow_steps[backref['ref']]
@@ -603,7 +783,7 @@ def setup_complete_task(test_case):
     assign_task(workers['entry'].id, task.id)
 
     task.refresh_from_db()
-    test_case.assertEquals(task.status, Task.Status.PROCESSING)
+    test_case.assertEqual(task.status, Task.Status.PROCESSING)
 
     submit_task(
         task.id, {'test': 'entry_submit'},
@@ -611,7 +791,7 @@ def setup_complete_task(test_case):
         workers['entry'])
 
     task.refresh_from_db()
-    test_case.assertEquals(task.status, Task.Status.PENDING_REVIEW)
+    test_case.assertEqual(task.status, Task.Status.PENDING_REVIEW)
 
     assign_task(workers['reviewer'].id, task.id)
     reviewer_assignment = task.assignments.get(
@@ -622,7 +802,7 @@ def setup_complete_task(test_case):
     reviewer_assignment.save()
 
     task.refresh_from_db()
-    test_case.assertEquals(task.status, Task.Status.REVIEWING)
+    test_case.assertEqual(task.status, Task.Status.REVIEWING)
 
     submit_task(
         task.id, {'test': 'reviewer_reject'},
@@ -630,7 +810,7 @@ def setup_complete_task(test_case):
         workers['reviewer'])
 
     task.refresh_from_db()
-    test_case.assertEquals(task.status, Task.Status.POST_REVIEW_PROCESSING)
+    test_case.assertEqual(task.status, Task.Status.POST_REVIEW_PROCESSING)
 
     submit_task(
         task.id, {'test': 'entry_resubmit'},
@@ -638,7 +818,7 @@ def setup_complete_task(test_case):
         workers['entry'])
 
     task.refresh_from_db()
-    test_case.assertEquals(task.status, Task.Status.REVIEWING)
+    test_case.assertEqual(task.status, Task.Status.REVIEWING)
 
     with patch('orchestra.utils.task_lifecycle._is_review_needed',
                return_value=False):
@@ -648,12 +828,12 @@ def setup_complete_task(test_case):
             workers['reviewer'])
 
     task.refresh_from_db()
-    test_case.assertEquals(task.status, Task.Status.COMPLETE)
-    test_case.assertEquals(task.assignments.count(), 2)
+    test_case.assertEqual(task.status, Task.Status.COMPLETE)
+    test_case.assertEqual(task.assignments.count(), 2)
     for assignment in task.assignments.all():
-        test_case.assertEquals(
+        test_case.assertEqual(
             assignment.status, TaskAssignment.Status.SUBMITTED)
-        test_case.assertEquals(assignment.iterations.count(), 2)
+        test_case.assertEqual(assignment.iterations.count(), 2)
 
     # Modify assignments with correct datetime
     new_datetime_labels = ('entry_pickup', 'reviewer_pickup')
