@@ -1,6 +1,7 @@
 from orchestra.models import Certification
-from orchestra.models import Workflow
+from orchestra.models import SanityCheck
 from orchestra.models import Step
+from orchestra.models import Workflow
 from orchestra.models import WorkflowVersion
 
 
@@ -13,6 +14,18 @@ def machine_task_function():
     pass
 
 
+def abort_cleanup_function(project, **kwargs):
+    pass
+
+
+def check_project(project):
+    return [
+        SanityCheck(check_slug='frequently_repeating_check'),
+        SanityCheck(check_slug='infrequently_repeating_check'),
+        SanityCheck(check_slug='onetime_check'),
+    ]
+
+
 workflow_fixtures = [
     {
         'slug': 'w1',
@@ -186,6 +199,9 @@ workflow_fixtures = [
                             'angular_module': 'step1',
                             'angular_directive': 'step1_directive',
                         },
+                        'todolist_templates_to_apply': [
+                            'project-checklist'
+                        ]
                     },
                     {
                         'slug': 'step_1',
@@ -218,6 +234,9 @@ workflow_fixtures = [
                             'angular_module': 'step2',
                             'angular_directive': 'step2_directive',
                         },
+                        'todolist_templates_to_apply': [
+                            'project-checklist'
+                        ]
                     },
                 ],
             },
@@ -243,7 +262,7 @@ workflow_fixtures = [
                         },
                         'is_human': True,
                         'creation_depends_on': [],
-                        'required_certifications': [],
+                        'required_certifications': ['certification1_ap'],
                         'review_policy': {
                             'policy': 'no_review',
                         },
@@ -272,12 +291,12 @@ workflow_fixtures = [
                             'path': ('orchestra.tests.helpers.'
                                      'fixtures.get_detailed_description'),
                             'kwargs': {
-                                'text': 'step 2 text',
+                                'text': 'step 1 text',
                             },
                         },
                         'is_human': True,
                         'creation_depends_on': ['staffbot_step_0'],
-                        'required_certifications': ['certification2_ap'],
+                        'required_certifications': ['certification1_ap'],
                         'review_policy': {
                             'policy': 'sampled_review',
                             'rate': 1,
@@ -299,8 +318,83 @@ workflow_fixtures = [
                         'user_interface': {
                             'javascript_includes': ['/path/to/some.js'],
                             'stylesheet_includes': ['/path/to/some.css'],
-                            'angular_module': 'step2',
-                            'angular_directive': 'step2_directive',
+                            'angular_module': 'step1',
+                            'angular_directive': 'step1_directive',
+                        },
+                    },
+                ],
+            },
+        ],
+    },
+    {
+        'slug': 'creation_policy',
+        'name': 'Creation Policy Workflow',
+        'versions': [
+            {
+                'slug': 'creation_policy_workflow',
+                'name': 'The workflow',
+                'description': 'A description of the workflow',
+                'steps': [
+                    {
+                        'slug': 'creation_policy_step_0',
+                        'name': 'The first step',
+                        'description': ('The longer description of the '
+                                        'first step'),
+                        'detailed_description_function': {
+                            'path': ('orchestra.tests.helpers.'
+                                     'fixtures.get_detailed_description')
+                        },
+                        'is_human': True,
+                        'creation_depends_on': [],
+                        'required_certifications': [],
+                        'review_policy': {
+                            'policy': 'no_review',
+                        },
+                        'creation_policy': {
+                            'policy_function': {
+                                'path': ('orchestra.creation_policies'
+                                         '.always_create'),
+                            }
+                        },
+                        'user_interface': {
+                            'javascript_includes': ['/path/to/some.js'],
+                            'stylesheet_includes': ['/path/to/some.css'],
+                            'angular_module': 'step1',
+                            'angular_directive': 'step1_directive',
+                        },
+                    },
+                    {
+                        'slug': 'creation_policy_step_1',
+                        'name': 'The second step',
+                        'description': ('The longer description of the '
+                                        'second step'),
+                        'detailed_description_function': {
+                            'path': ('orchestra.tests.helpers.'
+                                     'fixtures.get_detailed_description'),
+                            'kwargs': {
+                                'text': 'step 1 text',
+                            },
+                        },
+                        'is_human': True,
+                        'creation_depends_on': ['creation_policy_step_0'],
+                        'required_certifications': ['certification1_ap'],
+                        'review_policy': {
+                            'policy': 'sampled_review',
+                            'rate': 1,
+                            'max_reviews': 1,
+                        },
+                        'creation_policy': {
+                            'policy_function': {
+                                'path': ('orchestra.utils.tests.'
+                                         'test_creation_policies.never_create'
+                                         )
+                            }
+                        },
+                        'user_interface': {
+                            'javascript_includes': ['/path/to/some.js'],
+                            'stylesheet_includes': ['/path/to/some.css'],
+                            'angular_module': 'step1',
+                            'angular_directive': 'step1_directive',
                         },
                     },
                 ],
@@ -477,6 +571,103 @@ workflow_fixtures = [
             },
         ],
     },
+    # A workflow with a single human step with no review
+    {
+        'slug': 'single_step_human_workflow',
+        'name': 'Single step human workflow',
+        'versions': [
+            {
+                'slug': 'single_step_no_review',
+                'name': 'Single step no-review workflow',
+                'description': 'A workflow with a single step with no review',
+                'steps': [
+                    {
+                        'slug': 'step1',
+                        'name': 'Step 1',
+                        'description': 'Step 1',
+                        'is_human': True,
+                        'creation_depends_on': [],
+                        "review_policy": {
+                            "policy": "no_review"
+                        },
+                    }
+                ],
+            },
+        ],
+    },
+
+    {
+        'slug': 'sanitybot',
+        'name': 'SanityBot workflow',
+        'certifications': [],
+        'versions': [
+            {
+                'slug': 'sanitybot_workflow',
+                'name': 'SanityBot workflow',
+                'description': 'A description of the SanityBot workflow',
+                'sanity_checks': {
+                    'sanity_check_function': {
+                        'path': ('orchestra.tests.helpers.workflow'
+                                 '.check_project')
+                    },
+                    'check_configurations': {
+                        "frequently_repeating_check": {
+                            "handlers": [{
+                                "type": "slack_project_channel",
+                                "message": "Frequently repeating message",
+                                "steps": ["step1"]
+                            }],
+                            "repetition_seconds": 86400  # One day
+                        },
+                        "infrequently_repeating_check": {
+                            "handlers": [{
+                                "type": "slack_project_channel",
+                                "message": "Infrequently repeating message",
+                                "steps": ["step1"]
+                            }],
+                            "repetition_seconds": 864000  # Ten days
+                        },
+                        "onetime_check": {
+                            "handlers": [{
+                                "type": "slack_project_channel",
+                                "message": "One-time message",
+                                "steps": ["step1"]
+                            }],
+                        },
+                        "unused_check": {
+                            "handlers": [{
+                                "type": "slack_project_channel",
+                                "message": "Unused message",
+                                "steps": ["step1"]
+                            }],
+                        },
+                    }
+                },
+                'steps': [
+                    {
+                        'slug': 'step1',
+                        'name': 'The first step',
+                        'description': ('The longer description of the '
+                                        'first step'),
+                        'is_human': True,
+                        'creation_depends_on': [],
+                        'required_certifications': [],
+                        'review_policy': {
+                            'policy': 'sampled_review',
+                            'rate': 1,
+                            'max_reviews': 2,
+                        },
+                        'user_interface': {
+                            'javascript_includes': ['/path/to/some.js'],
+                            'stylesheet_includes': ['/path/to/some.css'],
+                            'angular_module': 'step1',
+                            'angular_directive': 'step1_directive',
+                        },
+                    },
+                ],
+            },
+        ],
+    },
 ]
 
 
@@ -552,7 +743,7 @@ def assert_test_dir_v1_not_loaded(test_case):
     })
 
 
-def assert_test_dir_v1_loaded(test_case):
+def assert_test_dir_v1_loaded(test_case, extra_step=False):
     """ Verify that the test_dir workflow v1's DB objects exist. """
     version1 = test_case.assertModelInstanceExists(
         WorkflowVersion,
@@ -634,6 +825,7 @@ def assert_test_dir_v1_loaded(test_case):
             'name': 'Step 3',
             'description': 'The third step',
             'is_human': True,
+            'completion_ends_project': True,
             'execution_function': {},
             'review_policy': {
                 'policy': 'sampled_review',
@@ -658,6 +850,42 @@ def assert_test_dir_v1_loaded(test_case):
     test_case.assertEqual(list(step3.creation_depends_on.all()), [step1])
     test_case.assertEqual(list(step3.submission_depends_on.all()), [step2])
 
+    if extra_step:
+        step4 = test_case.assertModelInstanceExists(
+            Step,
+            {
+                'workflow_version': version1,
+                'slug': 's4',
+            },
+            {
+                'name': 'Step 4',
+                'description': 'The fourth step',
+                'is_human': True,
+                'execution_function': {},
+                'review_policy': {
+                    'policy': 'sampled_review',
+                    'rate': 1,
+                    'max_reviews': 1,
+                },
+                'user_interface': {
+                    'angular_module': 'test_dir.v1.s3',
+                    'angular_directive': 's3',
+                    'javascript_includes': [
+                        'test_dir/v1/s3/js/modules.js',
+                        'test_dir/v1/s3/js/controllers.js',
+                        'test_dir/v1/s3/js/directives.js',
+                    ],
+                },
+            })
+        test_case.assertEqual(
+            set(step4.required_certifications.all()),
+            set(Certification.objects.filter(
+                slug__in=['certification1', 'certification2'],
+                workflow__slug='test_dir',
+            )))
+        test_case.assertEqual(list(step4.creation_depends_on.all()), [step1])
+        test_case.assertEqual(list(step4.submission_depends_on.all()), [step2])
+
 
 def assert_test_dir_v2_not_loaded(test_case):
     """ Verify that the test_dir workflow v2's DB objects exist. """
@@ -731,6 +959,10 @@ def assert_test_dir_v2_loaded(test_case):
                     'test_dir/v2/s2/js/directives.js',
                 ],
             },
+            'assignable_hours_function': {
+                'path': 'v2.machine.assignable_hours',
+                'kwargs': {}
+            },
         })
     test_case.assertEqual(list(step2.required_certifications.all()),
                           [Certification.objects.get(
