@@ -1,19 +1,22 @@
 import json
 
 from django.test import override_settings
-from orchestra.models import Task
-from orchestra.models import TaskAssignment
+
 from orchestra.models import Step
-from orchestra.tests.helpers import OrchestraTestCase
+from orchestra.models import Task
+from orchestra.models import Project
+from orchestra.models import TaskAssignment
+from orchestra.tests.helpers import OrchestraTransactionTestCase
 from orchestra.tests.helpers.fixtures import setup_models
 from orchestra.tests.helpers.iterations import verify_iterations
 from orchestra.utils.load_json import load_encoded_json
+from orchestra.utils.task_lifecycle import assign_task
 from orchestra.utils.task_lifecycle import create_subsequent_tasks
 
 
-class DashboardTestCase(OrchestraTestCase):
+class DashboardTestCase(OrchestraTransactionTestCase):
 
-    def setUp(self):  # noqa
+    def setUp(self):
         super().setUp()
         setup_models(self)
 
@@ -28,29 +31,26 @@ class DashboardTestCase(OrchestraTestCase):
         Task.objects.filter(status=Task.Status.AWAITING_PROCESSING).delete()
 
         project = self.projects['test_human_and_machine']
-        self.assertEquals(Task.objects.filter(project=project).count(),
-                          0)
+        self.assertEqual(Task.objects.filter(project=project).count(),
+                         0)
         create_subsequent_tasks(project)
 
         # Human Task was created
-        self.assertEquals(Task.objects.filter(project=project).count(),
-                          1)
-
-        response = (self.clients[0].get(
-            '/orchestra/api/interface/new_task_assignment/entry_level/'))
-        self.assertEquals(response.status_code, 200)
+        self.assertEqual(Task.objects.filter(project=project).count(),
+                         1)
 
         human_step = self.workflow_steps['test_workflow_2']['step4']
         task = Task.objects.get(step=human_step, project=project)
         data = {'submit_key1': 'submit_val1'}
+        assign_task(self.workers[0].id, task.id)
 
         # user 0 submits a task
         response = self._submit_assignment(self.clients[0], task.id, data=data)
-        self.assertEquals(response.status_code, 200)
+        self.assertEqual(response.status_code, 200)
 
         # Machine Task was created
-        self.assertEquals(Task.objects.filter(project=project).count(),
-                          2)
+        self.assertEqual(Task.objects.filter(project=project).count(),
+                         2)
         machine_step = self.workflow_steps['test_workflow_2']['simple_machine']
         machine_task_assignment = (
             TaskAssignment.objects
@@ -68,39 +68,42 @@ class DashboardTestCase(OrchestraTestCase):
 
     def test_index(self):
         response = self.clients[0].get('/orchestra/app/')
-        self.assertEquals(response.status_code, 200)
+        self.assertEqual(response.status_code, 200)
 
     def test_status(self):
         response = self.clients[0].get('/orchestra/status/')
-        self.assertEquals(response.status_code, 200)
+        self.assertEqual(response.status_code, 200)
 
     def test_dashboard_tasks(self):
         self._check_client_dashboard_state(self.clients[0], 'pending_review')
 
+    def test_dashboard_tasks_tags(self):
+        url = '/orchestra/api/interface/dashboard_tasks/'
+        response = self.clients[0].get(url)
+        returned = load_encoded_json(response.content)
+        self.assertEqual(len(returned['tasks'][0]['tags']), 0)
+
+        # set tags for this task
+        bad_formatted_tags = {'tags': [{'foo': 'bar'}]}
+        valid_tags = {'tags': [{'label': 'foo', 'status': 'default'}]}
+        task = TaskAssignment.objects.filter(
+            worker=self.workers[0])[0].task
+        with self.assertRaises(AttributeError):
+            task.tags = bad_formatted_tags
+            task.tags.save()
+
+        task.tags = valid_tags
+        task.save()
+        response2 = self.clients[0].get(url)
+        returned2 = load_encoded_json(response2.content)
+        tags = returned2['tasks'][0]['tags']
+        self.assertEqual(len(tags), 1)
+        self.assertEqual(tags[0]['label'], 'foo')
+        self.assertEqual(tags[0]['status'], 'default')
+
     def test_entry_level_task_assignment(self):
-        response = (self.clients[2].get(
-            '/orchestra/api/interface/new_task_assignment/entry_level/'))
-
-        # no task available for user3 because no entry-level certification
-        self.assertEquals(response.status_code, 400)
-        returned = load_encoded_json(response.content)
-        self.assertEquals(returned['message'],
-                          'No worker certificates')
-
-        # user 0 only has certification for entry level tasks
-        response = (self.clients[0].get(
-            '/orchestra/api/interface/new_task_assignment/entry_level/'))
-        self.assertEquals(response.status_code, 200)
-        returned = load_encoded_json(response.content)
-
-        task = Task.objects.get(id=returned['id'])
-        self.assertEquals({
-            'id': task.id,
-            'assignment_id': task.assignments.get(worker=self.workers[0]).id,
-            'step': task.step.slug,
-            'project': task.project.workflow_version.slug,
-            'detail': task.project.short_description
-        }, returned)
+        task = self.tasks['awaiting_processing']
+        assign_task(self.workers[0].id, task.id)
 
         # task assignment for invalid id should give bad request
         self._verify_bad_task_assignment_information(
@@ -119,43 +122,29 @@ class DashboardTestCase(OrchestraTestCase):
             'Processing', 'Processing', False,
             False, {}, self.workers[0])
 
-        # no more tasks for user 0 left
-        response = (self.clients[0].get(
-            '/orchestra/api/interface/new_task_assignment/entry_level/'))
-        self.assertEquals(response.status_code, 400)
-        returned = load_encoded_json(response.content)
-        self.assertEquals(returned['message'],
-                          'No task')
-
     def _check_client_dashboard_state(self, client, non_empty_status):
         response = client.get('/orchestra/api/interface/dashboard_tasks/')
         returned = load_encoded_json(response.content)
-        for status, val in returned['tasks'].items():
-            if status == non_empty_status:
-                self.assertTrue(len(val) > 0)
-            else:
-                self.assertTrue(len(val) == 0)
+        non_empty_tasks = [task for task in returned['tasks']
+                           if task['state'] == non_empty_status]
+        empty_tasks = [task for task in returned['tasks']
+                       if task['state'] != non_empty_status]
+        self.assertGreater(len(non_empty_tasks), 0)
+        self.assertEqual(len(empty_tasks), 0)
 
     def test_reviewer_task_assignment(self):
-        # there is a review task for user 1
-        response = (self.clients[1].get(
-            '/orchestra/api/interface/new_task_assignment/reviewer/'))
-        self.assertEquals(response.status_code, 200)
-
-        returned = load_encoded_json(response.content)
+        task = self.tasks['review_task']
+        assign_task(
+            self.workers[1].id, task.id)
         self._verify_good_task_assignment_information(
-            self.clients[1], {'task_id': returned['id']},
-            Task.objects.get(id=returned['id']).project.short_description,
+            self.clients[1], {'task_id': task.id},
+            task.project.short_description,
             'Processing', 'Reviewing', True,
             False, {'test_key': 'test_value'}, self.workers[1])
 
     def test_save_entry_level_task_assignment(self):
-        # user 0 only has certification for entry level tasks
-        response = (self.clients[0].get(
-            '/orchestra/api/interface/new_task_assignment/entry_level/'))
-        self.assertEquals(response.status_code, 200)
-        returned = load_encoded_json(response.content)
-        task = Task.objects.get(id=returned['id'])
+        task = self.tasks['awaiting_processing']
+        assign_task(self.workers[0].id, task.id)
 
         # incorrect task id
         response = self.clients[0].post(
@@ -163,10 +152,10 @@ class DashboardTestCase(OrchestraTestCase):
             json.dumps({'task_id': -1,
                         'task_data': 'test'}),
             content_type='application/json')
-        self.assertEquals(response.status_code, 400)
+        self.assertEqual(response.status_code, 400)
         returned = load_encoded_json(response.content)
-        self.assertEquals(returned['message'],
-                          'No task for given id')
+        self.assertEqual(returned['message'],
+                         'No task for given id')
 
         # user does not have a permission to save
         response = self.clients[1].post(
@@ -174,10 +163,10 @@ class DashboardTestCase(OrchestraTestCase):
             json.dumps({'task_id': task.id,
                         'task_data': 'test'}),
             content_type='application/json')
-        self.assertEquals(response.status_code, 400)
+        self.assertEqual(response.status_code, 400)
         returned = load_encoded_json(response.content)
-        self.assertEquals(returned['message'],
-                          'Worker is not associated with task')
+        self.assertEqual(returned['message'],
+                         'Worker is not associated with task')
 
         # save new info
         new_data = {'new_test_key': 'new_test_value'}
@@ -186,7 +175,7 @@ class DashboardTestCase(OrchestraTestCase):
             json.dumps({'task_id': task.id,
                         'task_data': new_data}),
             content_type='application/json')
-        self.assertEquals(response.status_code, 200)
+        self.assertEqual(response.status_code, 200)
 
         self._verify_good_task_assignment_information(
             self.clients[0], {'task_id': task.id},
@@ -196,15 +185,8 @@ class DashboardTestCase(OrchestraTestCase):
 
     def test_save_reviewer_task_assignment(self):
         new_data = {'new_test_key': 'new_test_value'}
-
-        # get a reviewer task
-        response = (self.clients[1].get(
-            '/orchestra/api/interface/new_task_assignment/reviewer/'))
-        self.assertEquals(response.status_code, 200)
-
-        returned = load_encoded_json(response.content)
-
-        task = Task.objects.get(id=returned['id'])
+        task = self.tasks['review_task']
+        assign_task(self.workers[1].id, task.id)
 
         # entry level worker can't update the data
         response = self.clients[0].post(
@@ -212,10 +194,10 @@ class DashboardTestCase(OrchestraTestCase):
             json.dumps({'task_id': task.id,
                         'task_data': new_data}),
             content_type='application/json')
-        self.assertEquals(response.status_code, 400)
+        self.assertEqual(response.status_code, 400)
         returned = load_encoded_json(response.content)
-        self.assertEquals(returned['message'],
-                          'Worker is not allowed to save')
+        self.assertEqual(returned['message'],
+                         'Worker is not allowed to save')
 
         reviewer_data = {'reviewer_key': 'reviewer_value'}
 
@@ -225,7 +207,7 @@ class DashboardTestCase(OrchestraTestCase):
             json.dumps({'task_id': task.id,
                         'task_data': reviewer_data}),
             content_type='application/json')
-        self.assertEquals(response.status_code, 200)
+        self.assertEqual(response.status_code, 200)
 
         self._verify_good_task_assignment_information(
             self.clients[1], {'task_id': task.id},
@@ -234,71 +216,67 @@ class DashboardTestCase(OrchestraTestCase):
             False, reviewer_data, self.workers[1])
 
     def test_submit_entry_level_task_assignment(self):
-        # user 0 only has certification for entry level tasks
-        response = (self.clients[0].get(
-            '/orchestra/api/interface/new_task_assignment/entry_level/'))
-        self.assertEquals(response.status_code, 200)
-        returned = load_encoded_json(response.content)
-        task = Task.objects.get(id=returned['id'])
+        task = self.tasks['awaiting_processing']
+        assign_task(self.workers[0].id, task.id)
 
         verify_iterations(task.id)
 
         # user is not assigned to a task
         response = self._submit_assignment(self.clients[1], task.id)
-        self.assertEquals(response.status_code, 400)
+        self.assertEqual(response.status_code, 400)
         returned = load_encoded_json(response.content)
-        self.assertEquals(returned['message'],
-                          'Task assignment with worker is in broken state.')
+        self.assertEqual(returned['message'],
+                         'Task assignment with worker is in broken state.')
 
         # task does not exist
         response = self._submit_assignment(self.clients[1], -1)
-        self.assertEquals(response.status_code, 400)
+        self.assertEqual(response.status_code, 400)
         returned = load_encoded_json(response.content)
-        self.assertEquals(returned['message'],
-                          'No task for given id')
+        self.assertEqual(returned['message'],
+                         'No task for given id')
 
         # user 0 can only submit a task not reject
         response = self._submit_assignment(
             self.clients[0], task.id, command='reject')
-        self.assertEquals(response.status_code, 400)
+        self.assertEqual(response.status_code, 400)
         returned = load_encoded_json(response.content)
-        self.assertEquals(returned['message'],
-                          'Task not in rejectable state.')
+        self.assertEqual(returned['message'],
+                         'Task not in rejectable state.')
 
         # user 0 can't call illegal commands
         response = self._submit_assignment(
             self.clients[0], task.id, command='approve')
-        self.assertEquals(response.status_code, 400)
+        self.assertEqual(response.status_code, 400)
         returned = load_encoded_json(response.content)
-        self.assertEquals(returned['message'],
-                          'Illegal command')
+        self.assertEqual(returned['message'],
+                         'Illegal command')
         data = {'submit_key1': 'submit_val1'}
 
         # user 0 can't submit a task if its submission prerequisites aren't
         # complete
         step = task.step
-        step.submission_depends_on = [
+        step.submission_depends_on.set([
             Step.objects.create(
                 workflow_version=step.workflow_version,
                 slug='imaginary_test_step',
                 is_human=True,
             )
-        ]
+        ])
         step.save()
         response = self._submit_assignment(
             self.clients[0], task.id)
-        self.assertEquals(response.status_code, 400)
+        self.assertEqual(response.status_code, 400)
         returned = load_encoded_json(response.content)
-        self.assertEquals(returned['message'],
-                          'Submission prerequisites are not complete.')
-        step.submission_depends_on = []
+        self.assertEqual(returned['message'],
+                         'Submission prerequisites are not complete.')
+        step.submission_depends_on.set([])
         step.save()
 
         data = {'submit_key1': 'submit_val1'}
         # user 0 submits a task
         response = self._submit_assignment(
             self.clients[0], task.id, data=data)
-        self.assertEquals(response.status_code, 200)
+        self.assertEqual(response.status_code, 200)
 
         self._verify_good_task_assignment_information(
             self.clients[0], {'task_id': task.id},
@@ -312,23 +290,15 @@ class DashboardTestCase(OrchestraTestCase):
         # user cannot resubmit a task
         response = self._submit_assignment(
             self.clients[0], task.id)
-        self.assertEquals(response.status_code, 400)
+        self.assertEqual(response.status_code, 400)
         returned = load_encoded_json(response.content)
-        self.assertEquals(returned['message'],
-                          'Worker is not allowed to submit')
+        self.assertEqual(returned['message'],
+                         'Worker is not allowed to submit')
 
     def test_submit_reviewer_task_assignment(self):
         data = {'submit_key1': 'submit_val1'}
-
-        # user 1 is picking up a task as a reviewer
-        response = (self.clients[1].get(
-            '/orchestra/api/interface/new_task_assignment/reviewer/'))
-        self.assertEquals(response.status_code, 200)
-        returned = load_encoded_json(response.content)
-        task_id = returned['id']
-        task = Task.objects.get(id=returned['id'])
-        self.assertEquals(task.assignments.count(), 2)
-        self.assertEquals(task_id, self.tasks['review_task'].id)
+        task = self.tasks['review_task']
+        assign_task(self.workers[1].id, task.id)
 
         verify_iterations(task.id)
 
@@ -350,8 +320,8 @@ class DashboardTestCase(OrchestraTestCase):
 
         # user 1 rejects a task
         response = self._submit_assignment(
-            self.clients[1], task_id, data=rejected_data, command='reject')
-        self.assertEquals(response.status_code, 200)
+            self.clients[1], task.id, data=rejected_data, command='reject')
+        self.assertEqual(response.status_code, 200)
 
         verify_iterations(task.id)
 
@@ -371,8 +341,8 @@ class DashboardTestCase(OrchestraTestCase):
 
         # user 0 submits an updated data
         response = self._submit_assignment(
-            self.clients[0], task_id, data=data)
-        self.assertEquals(response.status_code, 200)
+            self.clients[0], task.id, data=data)
+        self.assertEqual(response.status_code, 200)
 
         verify_iterations(task.id)
 
@@ -386,8 +356,8 @@ class DashboardTestCase(OrchestraTestCase):
         accepted_data = {'accepted_key': 'accepted_val'}
         # user 1 accepts a task
         response = self._submit_assignment(
-            self.clients[1], task_id, data=accepted_data, command='accept')
-        self.assertEquals(response.status_code, 200)
+            self.clients[1], task.id, data=accepted_data, command='accept')
+        self.assertEqual(response.status_code, 200)
 
         verify_iterations(task.id)
 
@@ -401,19 +371,14 @@ class DashboardTestCase(OrchestraTestCase):
 
         # make sure a task can't be submitted twice
         response = self._submit_assignment(
-            self.clients[1], task_id, command='accept')
-        self.assertEquals(response.status_code, 400)
+            self.clients[1], task.id, command='accept')
+        self.assertEqual(response.status_code, 400)
         returned = load_encoded_json(response.content)
-        self.assertEquals(returned['message'],
-                          'Worker is not allowed to submit')
+        self.assertEqual(returned['message'],
+                         'Worker is not allowed to submit')
 
         # user 3 is picking up a task as a reviewer
-        response = (self.clients[3].get(
-            '/orchestra/api/interface/new_task_assignment/reviewer/'))
-        self.assertEquals(response.status_code, 200)
-        returned = load_encoded_json(response.content)
-        self.assertEquals(returned['id'],
-                          task.id)
+        assign_task(self.workers[3].id, task.id)
 
         verify_iterations(task.id)
 
@@ -421,8 +386,8 @@ class DashboardTestCase(OrchestraTestCase):
 
         # user 3 rejects a task
         response = self._submit_assignment(
-            self.clients[3], task_id, data=rejected_data, command='reject')
-        self.assertEquals(response.status_code, 200)
+            self.clients[3], task.id, data=rejected_data, command='reject')
+        self.assertEqual(response.status_code, 200)
         returned = load_encoded_json(response.content)
 
         verify_iterations(task.id)
@@ -444,7 +409,7 @@ class DashboardTestCase(OrchestraTestCase):
 
         response = self._submit_assignment(
             self.clients[1], task.id)
-        self.assertEquals(response.status_code, 200)
+        self.assertEqual(response.status_code, 200)
 
         verify_iterations(task.id)
 
@@ -457,8 +422,8 @@ class DashboardTestCase(OrchestraTestCase):
 
         # check if the accepted_data is saved
         response = self._submit_assignment(
-            self.clients[3], task_id, data=accepted_data, command='accept')
-        self.assertEquals(response.status_code, 200)
+            self.clients[3], task.id, data=accepted_data, command='accept')
+        self.assertEqual(response.status_code, 200)
         returned = load_encoded_json(response.content)
 
         verify_iterations(task.id)
@@ -472,11 +437,11 @@ class DashboardTestCase(OrchestraTestCase):
 
         # check that reviewer cannot reaccept task
         response = self._submit_assignment(
-            self.clients[3], task_id, data=accepted_data, command='accept')
-        self.assertEquals(response.status_code, 400)
+            self.clients[3], task.id, data=accepted_data, command='accept')
+        self.assertEqual(response.status_code, 400)
         returned = load_encoded_json(response.content)
-        self.assertEquals(returned['message'],
-                          'Task already completed')
+        self.assertEqual(returned['message'],
+                         'Task already completed')
 
     def _verify_bad_task_assignment_information(
             self, client, post_data, error_message):
@@ -484,10 +449,10 @@ class DashboardTestCase(OrchestraTestCase):
             '/orchestra/api/interface/task_assignment_information/',
             json.dumps(post_data),
             content_type='application/json')
-        self.assertEquals(response.status_code, 400)
+        self.assertEqual(response.status_code, 400)
         returned = load_encoded_json(response.content)
-        self.assertEquals(returned['message'],
-                          error_message)
+        self.assertEqual(returned['message'],
+                         error_message)
 
     def _verify_good_task_assignment_information(
             self, client, post_data, project_description,
@@ -497,7 +462,7 @@ class DashboardTestCase(OrchestraTestCase):
             '/orchestra/api/interface/task_assignment_information/',
             json.dumps(post_data),
             content_type='application/json')
-        self.assertEquals(response.status_code, 200)
+        self.assertEqual(response.status_code, 200)
 
         task = Task.objects.get(id=post_data['task_id'])
         returned = load_encoded_json(response.content)
@@ -505,7 +470,9 @@ class DashboardTestCase(OrchestraTestCase):
             'project': {'details': project_description,
                         'id': task.project.id,
                         'project_data': {},
-                        'team_messages_url': None},
+                        'status': dict(
+                            Project.STATUS_CHOICES)[task.project.status],
+                        'scratchpad_url': None},
             'status': assignment_status,
             'task': {'data': task_data, 'status': task_status},
             'task_id': task.id,
@@ -519,6 +486,7 @@ class DashboardTestCase(OrchestraTestCase):
             'step': {'slug': 'step1', 'name': 'The first step'},
             'is_reviewer': is_reviewer,
             'is_read_only': is_read_only,
+            'is_project_admin': False,
             'worker': {
                 'username': worker.user.username,
                 'first_name': worker.user.first_name,
@@ -526,4 +494,4 @@ class DashboardTestCase(OrchestraTestCase):
             }
         }
 
-        self.assertEquals(returned, expected)
+        self.assertEqual(returned, expected)
