@@ -1,32 +1,44 @@
+import logging
 import random
 from pydoc import locate
 
 from django.conf import settings
+from django.db import connection
 from django.db import transaction
+from django.db.models import Case
+from django.db.models import When
+from django.db.models import Q
+from django.db.models import Value
+from django.db.models import IntegerField
 from django.utils import timezone
 
+
+from orchestra.communication.slack import add_worker_to_project_team
+from orchestra.communication.slack import archive_project_slack_group
+from orchestra.communication.utils import mark_worker_as_winner
 from orchestra.core.errors import AssignmentPolicyError
+from orchestra.core.errors import CreationPolicyError
 from orchestra.core.errors import IllegalTaskSubmission
-from orchestra.core.errors import ModelSaveError
-from orchestra.core.errors import NoTaskAvailable
 from orchestra.core.errors import ReviewPolicyError
 from orchestra.core.errors import TaskAssignmentError
 from orchestra.core.errors import TaskDependencyError
 from orchestra.core.errors import TaskStatusError
+from orchestra.core.errors import ProjectStatusError
 from orchestra.core.errors import WorkerCertificationError
 from orchestra.models import Iteration
 from orchestra.models import Project
 from orchestra.models import Task
+from orchestra.models import Todo
 from orchestra.models import TaskAssignment
 from orchestra.models import Worker
 from orchestra.models import WorkerCertification
-from orchestra.communication.slack import add_worker_to_project_team
+from orchestra.todos.api import add_todolist_template
 from orchestra.utils.notifications import notify_status_change
+from orchestra.utils.notifications import notify_project_status_change
 from orchestra.utils.task_properties import assignment_history
 from orchestra.utils.task_properties import current_assignment
 from orchestra.utils.task_properties import get_latest_iteration
 
-import logging
 logger = logging.getLogger(__name__)
 
 
@@ -100,7 +112,8 @@ def worker_has_reviewer_status(worker,
 
 
 def is_worker_certified_for_task(
-        worker, task, role, task_class=WorkerCertification.TaskClass.REAL):
+        worker, task, role, task_class=WorkerCertification.TaskClass.REAL,
+        require_staffbot_enabled=False):
     """
     Check whether worker is certified for a given task, role, and task
     class.
@@ -112,6 +125,8 @@ def is_worker_certified_for_task(
             The specified task object.
         task_class (orchestra.models.WorkerCertification.TaskClass):
             The specified task class.
+        require_staffbot_enabled (bool):
+            Whether to require that the `staffbot_enabled` flag be `True`.
 
     Returns:
         certified_for_task (bool):
@@ -119,15 +134,19 @@ def is_worker_certified_for_task(
             class.
     """
     step = task.step
-    match_count = (
+
+    worker_certifications = (
         WorkerCertification
         .objects
         .filter(worker=worker,
                 role=role,
                 task_class=task_class,
-                certification__in=step.required_certifications.all())
-        .count())
-    certified_for_task = step.required_certifications.count() == match_count
+                certification__in=step.required_certifications.all()))
+    if require_staffbot_enabled:
+        worker_certifications = worker_certifications.filter(
+            staffbot_enabled=True)
+    certified_for_task = (
+        step.required_certifications.count() == worker_certifications.count())
     return certified_for_task
 
 
@@ -170,7 +189,7 @@ def role_required_for_new_task(task):
 
 
 @transaction.atomic
-def assign_task(worker_id, task_id):
+def assign_task(worker_id, task_id, staffing_request_inquiry=None):
     """
     Return a given task after assigning or reassigning it to the specified
     worker.
@@ -195,6 +214,7 @@ def assign_task(worker_id, task_id):
     worker = Worker.objects.get(id=worker_id)
     task = Task.objects.get(id=task_id)
     required_role = role_required_for_new_task(task)
+    required_role_counter = role_counter_required_for_new_task(task)
 
     assignment = current_assignment(task)
     if not is_worker_certified_for_task(worker, task, required_role):
@@ -229,12 +249,15 @@ def assign_task(worker_id, task_id):
         start_datetime=assignment.start_datetime)
 
     add_worker_to_project_team(worker, task.project)
-    notify_status_change(task, previous_status)
+    notify_status_change(task, previous_status, staffing_request_inquiry)
+    mark_worker_as_winner(worker, task, required_role_counter,
+                          staffing_request_inquiry)
     return task
 
 
 @transaction.atomic
-def reassign_assignment(worker_id, assignment_id):
+def reassign_assignment(worker_id, assignment_id,
+                        staffing_request_inquiry=None):
     """
     Return a given assignment after reassigning it to the specified
     worker.
@@ -270,6 +293,10 @@ def reassign_assignment(worker_id, assignment_id):
 
     assignment.worker = worker
     assignment.save()
+
+    mark_worker_as_winner(worker, assignment.task,
+                          assignment.assignment_counter,
+                          staffing_request_inquiry)
     add_worker_to_project_team(worker, assignment.task.project)
     return assignment
 
@@ -277,7 +304,9 @@ def reassign_assignment(worker_id, assignment_id):
 @transaction.atomic
 def complete_and_skip_task(task_id):
     """
-    Marks a task and its assignments as complete and creates subsequent tasks.
+    Submits a task on behalf of the worker working on it. If the task
+    isn't assigned to a worker, marks it and its assignments as
+    complete and creates subsequent tasks.
 
     Args:
         task_id (int):
@@ -286,14 +315,24 @@ def complete_and_skip_task(task_id):
     Returns:
         task (orchestra.models.Task):
             The completed and skipped task.
+
     """
     task = Task.objects.get(id=task_id)
-    task.status = Task.Status.COMPLETE
-    task.save()
-    for assignment in task.assignments.all():
-        assignment.status = TaskAssignment.Status.SUBMITTED
-        assignment.save()
-    create_subsequent_tasks(task.project)
+    assignment = current_assignment(task)
+    if assignment and assignment.worker:
+        if assignment.status != TaskAssignment.Status.PROCESSING:
+            return task
+        task_data = assignment.in_progress_task_data or {}
+        task_data.update(_orchestra_internal={'complete_and_skip_task': True})
+        submit_task(
+            task_id, task_data,
+            Iteration.Status.REQUESTED_REVIEW, assignment.worker)
+    else:
+        task.status = Task.Status.COMPLETE
+        task.save()
+        for assignment in task.assignments.all():
+            assignment.status = TaskAssignment.Status.SUBMITTED
+            assignment.save()
     return task
 
 
@@ -310,9 +349,10 @@ def get_task_details(task_id):
     task = Task.objects.select_related('step__workflow_version__workflow').get(
         id=task_id)
     step = task.step
+    project = task.project
     workflow_version = step.workflow_version
     workflow = workflow_version.workflow
-    prerequisites = previously_completed_task_data(task)
+    prerequisites = get_previously_completed_task_data(step, project)
 
     task_details = {
         'workflow': {
@@ -329,10 +369,11 @@ def get_task_details(task_id):
         },
         'task_id': task.id,
         'project': {
-            'id': task.project.id,
-            'details': task.project.short_description,
-            'team_messages_url': task.project.team_messages_url,
-            'project_data': task.project.project_data
+            'id': project.id,
+            'details': project.short_description,
+            'scratchpad_url': project.scratchpad_url,
+            'project_data': project.project_data,
+            'status': dict(Project.STATUS_CHOICES)[project.status]
         },
         'prerequisites': prerequisites
     }
@@ -393,12 +434,17 @@ def get_task_overview_for_worker(task_id, worker):
             Information about `task` and its assignment for `worker`.
     """
     task = Task.objects.get(id=task_id)
-    if not task.is_worker_assigned(worker):
-        raise TaskAssignmentError('Worker is not associated with task')
     task_details = get_task_details(task_id)
+    task_details['is_project_admin'] = worker.is_project_admin()
+
+    if task.is_worker_assigned(worker):
+        task_assignment = TaskAssignment.objects.get(worker=worker, task=task)
+    elif worker.is_project_admin():
+        task_assignment = assignment_history(task).last()
+        task_details['is_read_only'] = True
+    else:
+        raise TaskAssignmentError('Worker is not associated with task')
 
-    task_assignment = TaskAssignment.objects.get(worker=worker,
-                                                 task=task)
     task_assignment_details = get_task_assignment_details(task_assignment)
     task_assignment_details.update(task_details)
     return task_assignment_details
@@ -426,6 +472,9 @@ def tasks_assigned_to_worker(worker):
         .filter(
             worker=worker,
             status=TaskAssignment.Status.PROCESSING)
+        .exclude(
+            task__project__status__in=[Project.Status.PAUSED,
+                                       Project.Status.COMPLETED])
         .order_by('-task__project__priority',
                   'task__project__start_datetime'))
 
@@ -435,7 +484,11 @@ def tasks_assigned_to_worker(worker):
             worker=worker,
             status=TaskAssignment.Status.SUBMITTED
         )
-        .exclude(task__status=Task.Status.COMPLETE)
+        .exclude(
+            task__status=Task.Status.COMPLETE)
+        .exclude(
+            task__project__status__in=[Project.Status.PAUSED,
+                                       Project.Status.COMPLETED])
         .order_by('-task__project__priority',
                   'task__project__start_datetime'))
 
@@ -454,15 +507,24 @@ def tasks_assigned_to_worker(worker):
             inactive_review_task_assignments.append(task_assignment)
 
     # TODO(marcua): Do a better job of paginating than cutting off to the most
-    # recent 20 tasks.
+    # recent 200 tasks.
     complete_task_assignments = (
         valid_task_assignments
-        .filter(worker=worker,
-                task__status=Task.Status.COMPLETE)
+        .filter(Q(worker=worker) &
+                (Q(task__status=Task.Status.COMPLETE) |
+                 Q(task__project__status=Project.Status.COMPLETED)))
         .order_by('-task__project__priority',
-                  '-task__project__start_datetime')[:20])
+                  '-task__project__start_datetime')[:200])
+
+    paused_task_assignments = (
+        valid_task_assignments
+        .filter(
+            worker=worker,
+            status=TaskAssignment.Status.PROCESSING,
+            task__project__status=Project.Status.PAUSED)
+        .order_by('-task__project__priority',
+                  'task__project__start_datetime'))
 
-    # TODO(jrbotros): convert this to a filterable list
     task_assignments_overview = {
         'returned': (
             active_task_assignments
@@ -472,23 +534,109 @@ def tasks_assigned_to_worker(worker):
             .exclude(task__status=Task.Status.POST_REVIEW_PROCESSING)),
         'pending_review': inactive_review_task_assignments,
         'pending_processing': inactive_processing_task_assignments,
+        'paused': paused_task_assignments,
         'complete': complete_task_assignments}
 
-    tasks_assigned = {}
+    tasks_assigned = []
+    time_now = timezone.now()
+    pending_todos_filter = Q(status=Todo.Status.PENDING.value)
+    non_template_todo_filter = Q(template__isnull=True)
+    no_section_todo_filter = Q(section__isnull=True) | Q(section='')
     for state, task_assignments in iter(task_assignments_overview.items()):
-        tasks_val = []
         for task_assignment in task_assignments:
             step = task_assignment.task.step
             workflow_version = step.workflow_version
+            next_todo_title = None
+            next_todo_dict = {}
+            should_be_active = False
+            if state in ('returned', 'in_progress'):
+                # TODO(aditya): Temporarily we are filtering out todos
+                # with section values. Remove this comment once we
+                # figure out a long term logic.
+                next_todo = (
+                    task_assignment.task.todos
+                    .filter(
+                        pending_todos_filter &
+                        non_template_todo_filter &
+                        no_section_todo_filter
+                    ).annotate(
+                        todo_order=Case(
+                            When(
+                                start_by_datetime__gt=time_now,
+                                then=Value(3)),
+                            When(
+                                due_datetime=None,
+                                then=Value(2)),
+                            default=Value(1),
+                            output_field=IntegerField()
+                        )
+                    )
+                    .order_by(
+                        'todo_order',
+                        'due_datetime',
+                        'start_by_datetime',
+                        '-created_at'
+                    )
+                    .first())
 
-            # TODO(jrbotros): standardize task/assignment serialization
-            tasks_val.append({
+                if next_todo:
+                    next_todo_title = next_todo.title
+                    start_str = (
+                        next_todo.start_by_datetime.strftime(
+                            '%Y-%m-%dT%H:%M:%SZ'
+                        ) if next_todo.start_by_datetime else ''
+                    )
+                    due_str = (
+                        next_todo.due_datetime.strftime(
+                            '%Y-%m-%dT%H:%M:%SZ'
+                        ) if next_todo.due_datetime else ''
+                    )
+                    next_todo_dict = {
+                        'title': next_todo.title,
+                        'start_by_datetime': start_str,
+                        'due_datetime': due_str
+                    }
+                # TODO(aditya): Temporarily we are filtering out todos
+                # with section values. Remove this comment once we
+                # figure out a long term logic.
+                num_non_template_todos = (
+                    task_assignment.task.todos
+                    .filter(
+                        non_template_todo_filter &
+                        no_section_todo_filter
+                    ).count())
+                # If a task has no todos (complete or incomplete)
+                # assigned to it, then by default the task would be
+                # marked as pending. When a task is first created and
+                # picked up by a worker, it will thus be in pending
+                # state, which is confusing behavior. We thus treat a
+                # task with zero todos as active. After a task has one
+                # or more todos assigned to it, its active/pending
+                # state is determined by the presence of incomplete
+                # todos.
+                task_started = (
+                    next_todo_title is not None
+                    and (
+                        next_todo.start_by_datetime is None
+                        or next_todo.start_by_datetime <= time_now
+                    )
+                )
+                should_be_active = (
+                    (num_non_template_todos == 0)
+                    or task_started)
+            tasks_assigned.append({
                 'id': task_assignment.task.id,
                 'assignment_id': task_assignment.id,
                 'step': step.name,
                 'project': workflow_version.name,
-                'detail': task_assignment.task.project.short_description})
-        tasks_assigned[state] = tasks_val
+                'detail': task_assignment.task.project.short_description,
+                'priority': task_assignment.task.project.priority,
+                'state': state,
+                'assignment_start_datetime': task_assignment.start_datetime,
+                'next_todo_dict': next_todo_dict,
+                'should_be_active': should_be_active,
+                'tags': task_assignment.task.tags.get('tags', [])
+            })
     return tasks_assigned
 
 
@@ -595,7 +743,8 @@ def check_worker_allowed_new_assignment(worker):
         orchestra.core.errors.TaskStatusError:
             New task assignment is not permitted for the given status.
     """
-    if worker_assigned_to_rejected_task(worker):
+    if (worker_assigned_to_rejected_task(worker) and
+            settings.ORCHESTRA_ENFORCE_NO_NEW_TASKS_DURING_REVIEW):
         raise TaskAssignmentError('Worker has pending reviewer feedback that '
                                   'must be addressed.')
     elif worker_assigned_to_max_tasks(worker):
@@ -609,56 +758,6 @@ def assert_new_task_status_valid(task_status):
         raise TaskStatusError('Invalid status for new task assignment.')
 
 
-def get_new_task_assignment(worker, task_status):
-    """
-    Check if new task assignment is available for the provided worker
-    and task status; if so, assign the task to the worker and return the
-    assignment.
-
-    Args:
-        worker (orchestra.models.Worker):
-            The worker submitting the task.
-        task_status (orchestra.models.Task.Status):
-            The status of the desired new task assignment.
-
-    Returns:
-        assignment (orchestra.models.TaskAssignment):
-            The newly created task assignment.
-
-    Raises:
-        orchestra.core.errors.WorkerCertificationError:
-            No human tasks are available for the given task status
-            except those for which the worker is not certified.
-        orchestra.core.errors.NoTaskAvailable:
-            No human tasks are available for the given task status.
-    """
-    assert_new_task_status_valid(task_status)
-    check_worker_allowed_new_assignment(worker)
-
-    tasks = (Task.objects
-             .filter(status=task_status)
-             .exclude(assignments__worker=worker)
-             .order_by('-project__priority')
-             .order_by('project__start_datetime'))
-
-    certification_error = False
-    for task in tasks.iterator():
-        try:
-            task = assign_task(worker.id, task.id)
-            return current_assignment(task)
-        except WorkerCertificationError:
-            certification_error = True
-        except ModelSaveError:
-            # Machine task cannot have human worker; treat machine tasks as if
-            # they do not exist
-            pass
-
-    if certification_error:
-        raise WorkerCertificationError
-    else:
-        raise NoTaskAvailable('No task available for {}'.format(worker))
-
-
 @transaction.atomic
 def save_task(task_id, task_data, worker):
     """
@@ -686,7 +785,7 @@ def save_task(task_id, task_data, worker):
         raise TaskAssignmentError('Worker is not associated with task')
 
     # Use select_for_update to prevent concurrency issues with submit_task.
-    # See https://github.com/unlimitedlabs/orchestra/issues/2.
+    # See https://github.com/b12io/orchestra/issues/2.
     assignment = (TaskAssignment.objects.select_for_update()
                                 .get(task=task, worker=worker))
 
@@ -780,7 +879,7 @@ def submit_task(task_id, task_data, iteration_status, worker):
         raise TaskStatusError('Task already completed')
 
     # Use select_for_update to prevent concurrency issues with save_task.
-    # See https://github.com/unlimitedlabs/orchestra/issues/2.
+    # See https://github.com/b12io/orchestra/issues/2.
     assignments = (TaskAssignment.objects.select_for_update()
                                  .filter(worker=worker, task=task))
 
@@ -831,22 +930,22 @@ def submit_task(task_id, task_data, iteration_status, worker):
     return task
 
 
-def previously_completed_task_data(task):
+def get_previously_completed_task_data(step, project):
     """
     Returns a dict mapping task prerequisites onto their
     latest task assignment information.  The dict is of the form:
     {'previous-slug': {latest_assignment_data}, ...}
 
     Args:
-        task (orchestra.models.Task): The specified task object.
-
+        step (orchestra.models.Step): The specified step object.
+        project (orchestra.models.Project): The specified project object.
     Returns:
         prerequisites (dict):
             A dict mapping task prerequisites onto their latest task
             assignment information.
     """
     # Find all prerequisite steps in graph
-    to_visit = list(task.step.creation_depends_on.all())
+    to_visit = list(step.creation_depends_on.all())
     prerequisite_steps = set(to_visit)
     while to_visit:
         current_step = to_visit.pop()
@@ -857,7 +956,7 @@ def previously_completed_task_data(task):
 
     prerequisite_data = {}
     for step in prerequisite_steps:
-        required_task = Task.objects.get(step=step, project=task.project)
+        required_task = Task.objects.get(step=step, project=project)
 
         if required_task.status != Task.Status.COMPLETE:
             raise TaskDependencyError('Task depenency is not satisfied')
@@ -896,6 +995,17 @@ def update_related_assignment_status(task, assignment_counter, data,
         start_datetime=submit_datetime)
 
 
+def _call_abort_completion_function(project):
+    abort_completion_fn = project.workflow_version.abort_completion_function
+    abort_completion_fn_path = abort_completion_fn.get('path', '')
+    abort_completion_fn_kwargs = abort_completion_fn.get('kwargs', {})
+    if abort_completion_fn_path:
+        abort_completion_function = locate(abort_completion_fn_path)
+        abort_completion_function(
+            project,
+            **abort_completion_fn_kwargs)
+
+
 def end_project(project_id):
     """
     Mark the specified project and its component tasks as aborted.
@@ -909,10 +1019,77 @@ def end_project(project_id):
     project = Project.objects.get(id=project_id)
     project.status = Project.Status.ABORTED
     project.save()
+    _call_abort_completion_function(project)
     for task in project.tasks.all():
         task.status = Task.Status.ABORTED
         task.save()
         notify_status_change(task, assignment_history(task))
+        notify_project_status_change(project)
+    archive_project_slack_group(project)
+
+
+def set_project_status(project_id, status):
+    """
+    Set the project to the given status. Raise an exception if the status
+    is not a valid project status.
+
+    Args:
+        project_id (int): The ID of the project
+        status (str): The new project status
+
+    Returns:
+        None
+
+    Raises:
+        orchestra.core.errors.ProjectStatusError:
+            The specified new status is not supported.
+    """
+    project = Project.objects.get(id=project_id)
+    status_choices = dict(Project.STATUS_CHOICES)
+    if status == status_choices[Project.Status.PAUSED]:
+        project.status = Project.Status.PAUSED
+        notify_project_status_change(project)
+    elif status == status_choices[Project.Status.ACTIVE]:
+        project.status = Project.Status.ACTIVE
+        notify_project_status_change(project)
+    elif status == status_choices[Project.Status.COMPLETED]:
+        project.status = Project.Status.COMPLETED
+        notify_project_status_change(project)
+    elif status == status_choices[Project.Status.ABORTED]:
+        raise ProjectStatusError((
+            'Try aborting the project with set_project_status. '
+            'Use end_project instead.'))
+    else:
+        raise ProjectStatusError('Invalid project status.')
+    project.save()
+
+
+def _check_creation_policy(step, project):
+    """
+    Check the creation policy of the given step, which will determine whether
+    to create the task.
+
+    Args:
+        step (orchestra.models.Step)
+        project (orchestra.models.Project)
+    Returns:
+        success (boolean) if the task should be created.
+    Raises:
+        orchestra.core.errors.CreationPolicyError:
+            The specified creation policy type is not supported or the policy
+            failed during execution.
+    """
+    policy = step.creation_policy.get('policy_function', {})
+    policy_path = policy.get('path', '')
+    policy_kwargs = policy.get('kwargs', {})
+    try:
+        policy_function = locate(policy_path)
+        prerequisite_data = get_previously_completed_task_data(step, project)
+        project_data = project.project_data
+        return policy_function(
+            prerequisite_data, project_data, **policy_kwargs)
+    except Exception as e:
+        raise CreationPolicyError(e)
 
 
 def _preassign_workers(task, policy_type):
@@ -956,7 +1133,18 @@ def _preassign_workers(task, policy_type):
     return task
 
 
+def schedule_machine_tasks(project, steps):
+    machine_step_scheduler_class = locate(
+        settings.MACHINE_STEP_SCHEDULER['path']
+    )
+    kwargs = settings.MACHINE_STEP_SCHEDULER.get('kwargs', {})
+    machine_step_scheduler = machine_step_scheduler_class(**kwargs)
+    for step in steps:
+        machine_step_scheduler.schedule(project.id, step.slug)
+
+
 # TODO(kkamalov): make a periodic job that runs this function periodically
+@transaction.atomic
 def create_subsequent_tasks(project):
     """
     Create tasks for a given project whose dependencies have been
@@ -971,31 +1159,51 @@ def create_subsequent_tasks(project):
             The modified project object.
     """
     workflow_version = project.workflow_version
-    all_steps = workflow_version.steps.all()
+    all_steps = workflow_version.steps.all().order_by('id')
 
     # get all completed tasks associated with a given project
     completed_tasks = Task.objects.filter(status=Task.Status.COMPLETE,
                                           project=project)
     completed_step_slugs = set(completed_tasks.values_list('step__slug',
                                                            flat=True))
+
+    end_project = False
+    machine_tasks_to_schedule = []
     for step in all_steps:
+        end_project = end_project or (
+            step.slug in completed_step_slugs and step.completion_ends_project)
+
         if step.slug in completed_step_slugs or Task.objects.filter(
                 project=project, step=step).exists():
             continue
 
         if _are_desired_steps_completed_on_project(
                 step.creation_depends_on, completed_tasks=completed_tasks):
-            # create new task and task_assignment
-            task = Task(step=step,
-                        project=project,
-                        status=Task.Status.AWAITING_PROCESSING)
-            task.save()
+            if _check_creation_policy(step, project):
+                # create new task and task_assignment
+                task = Task(step=step,
+                            project=project,
+                            status=Task.Status.AWAITING_PROCESSING)
+                task.save()
 
-            _preassign_workers(task, AssignmentPolicyType.ENTRY_LEVEL)
-            if not step.is_human:
-                machine_step_scheduler_class = locate(
-                    settings.MACHINE_STEP_SCHEDULER['path']
-                )
-                kwargs = settings.MACHINE_STEP_SCHEDULER.get('kwargs', {})
-                machine_step_scheduler = machine_step_scheduler_class(**kwargs)
-                machine_step_scheduler.schedule(project.id, step.slug)
+                # Apply todolist templates to Task
+                for template in task.step.todolist_templates_to_apply.all():
+                    add_todolist_template(template.slug, project.id, step.slug)
+
+                _preassign_workers(task, AssignmentPolicyType.ENTRY_LEVEL)
+
+                if not step.is_human:
+                    machine_tasks_to_schedule.append(step)
+
+    if len(machine_tasks_to_schedule) > 0:
+        connection.on_commit(lambda: schedule_machine_tasks(
+            project, machine_tasks_to_schedule))
+
+    incomplete_tasks = (Task.objects.filter(project=project)
+                        .exclude(Q(status=Task.Status.COMPLETE) |
+                                 Q(status=Task.Status.ABORTED)))
+
+    if end_project or incomplete_tasks.count() == 0:
+        if project.status != Project.Status.COMPLETED:
+            set_project_status(project.id, 'Completed')
+            archive_project_slack_group(project)
