@@ -1,9 +1,13 @@
-from unittest.mock import patch
 from unittest.mock import MagicMock
+from unittest.mock import patch
+
+from dateutil.parser import parse
+
+from django.contrib.auth import get_user_model
 
-from orchestra.core.errors import IllegalTaskSubmission
-from orchestra.core.errors import NoTaskAvailable
 from orchestra.core.errors import AssignmentPolicyError
+from orchestra.core.errors import CreationPolicyError
+from orchestra.core.errors import IllegalTaskSubmission
 from orchestra.core.errors import ModelSaveError
 from orchestra.core.errors import ReviewPolicyError
 from orchestra.core.errors import TaskAssignmentError
@@ -13,44 +17,72 @@ from orchestra.models import Iteration
 from orchestra.models import Project
 from orchestra.models import Task
 from orchestra.models import TaskAssignment
+from orchestra.models import Worker
 from orchestra.models import WorkerCertification
-from orchestra.tests.helpers import OrchestraTestCase
-from orchestra.tests.helpers.fixtures import setup_models
+from orchestra.tests.helpers import OrchestraTransactionTestCase
 from orchestra.tests.helpers.fixtures import StepFactory
 from orchestra.tests.helpers.fixtures import TaskAssignmentFactory
 from orchestra.tests.helpers.fixtures import TaskFactory
+from orchestra.tests.helpers.fixtures import TodoFactory
+from orchestra.tests.helpers.fixtures import ProjectFactory
+from orchestra.tests.helpers.fixtures import setup_models
+from orchestra.tests.helpers.fixtures import WorkflowVersionFactory
 from orchestra.utils.task_lifecycle import AssignmentPolicyType
-from orchestra.utils.task_lifecycle import is_worker_certified_for_task
+from orchestra.utils.task_lifecycle import assert_new_task_status_valid
 from orchestra.utils.task_lifecycle import assign_task
 from orchestra.utils.task_lifecycle import create_subsequent_tasks
-from orchestra.utils.task_lifecycle import get_new_task_assignment
 from orchestra.utils.task_lifecycle import get_next_task_status
 from orchestra.utils.task_lifecycle import get_task_overview_for_worker
+from orchestra.utils.task_lifecycle import is_worker_certified_for_task
 from orchestra.utils.task_lifecycle import role_counter_required_for_new_task
 from orchestra.utils.task_lifecycle import submit_task
+from orchestra.utils.task_lifecycle import tasks_assigned_to_worker
 from orchestra.utils.task_lifecycle import worker_assigned_to_rejected_task
 from orchestra.utils.task_lifecycle import worker_has_reviewer_status
+from orchestra.utils.task_lifecycle import end_project
 from orchestra.utils.task_properties import current_assignment
+from orchestra.workflow.defaults import get_default_creation_policy
+
+MOCK_CURRENT = '2018-01-17T00:00:00Z'
+DEADLINE1_DATETIME = '2018-01-18T00:00:00Z'
+DEADLINE2_DATETIME = '2018-01-19T00:00:00Z'
 
 
-class BasicTaskLifeCycleTestCase(OrchestraTestCase):
+class BasicTaskLifeCycleTestCase(OrchestraTransactionTestCase):
     """
     Test modular functions in the task_lifecycle
     """
 
-    def setUp(self):  # noqa
+    def setUp(self):
         super().setUp()
         setup_models(self)
+        self.workflow_version = WorkflowVersionFactory()
+        self.step = StepFactory(
+            slug='step-slug',
+            workflow_version=self.workflow_version)
+        self.project = ProjectFactory(
+            workflow_version=self.workflow_version)
 
     def test_is_worker_certified_for_task(self):
         task = Task.objects.filter(status=Task.Status.AWAITING_PROCESSING)[0]
 
+        for worker_certification in (
+                WorkerCertification.objects.filter(worker=self.workers[0])):
+            worker_certification.staffbot_enabled = False
+            worker_certification.save()
+
         # workers[0] has a certification
         self.assertTrue(
             is_worker_certified_for_task(self.workers[0],
                                          task,
                                          WorkerCertification.Role.ENTRY_LEVEL))
 
+        self.assertFalse(
+            is_worker_certified_for_task(self.workers[0],
+                                         task,
+                                         WorkerCertification.Role.ENTRY_LEVEL,
+                                         require_staffbot_enabled=True))
+
         # workers[2] has no certification
         self.assertFalse(
             is_worker_certified_for_task(self.workers[2],
@@ -58,6 +90,8 @@ class BasicTaskLifeCycleTestCase(OrchestraTestCase):
                                          WorkerCertification.Role.ENTRY_LEVEL))
 
     def test_not_allowed_new_assignment(self):
+        assert_new_task_status_valid(Task.Status.AWAITING_PROCESSING)
+        assert_new_task_status_valid(Task.Status.PENDING_REVIEW)
         invalid_statuses = [Task.Status.PROCESSING,
                             Task.Status.REVIEWING,
                             Task.Status.POST_REVIEW_PROCESSING,
@@ -65,77 +99,7 @@ class BasicTaskLifeCycleTestCase(OrchestraTestCase):
                             Task.Status.ABORTED]
         for status in invalid_statuses:
             with self.assertRaises(TaskStatusError):
-                get_new_task_assignment(self.workers[2], status)
-
-    def test_get_new_task_assignment_entry_level(self):
-        # Entry-level assignment
-        self.assertEquals(Task.objects
-                          .filter(status=Task.Status.AWAITING_PROCESSING)
-                          .count(),
-                          1)
-
-        with self.assertRaises(WorkerCertificationError):
-            get_new_task_assignment(self.workers[5],
-                                    Task.Status.PENDING_REVIEW)
-
-        # assign a new task to a worker
-        assignment = get_new_task_assignment(self.workers[5],
-                                             Task.Status.AWAITING_PROCESSING)
-        self.assertTrue(assignment is not None)
-
-        self.assertEquals(assignment.task.status,
-                          Task.Status.PROCESSING)
-
-        # No more tasks left in AWAITING_PROCESSING
-        with self.assertRaises(NoTaskAvailable):
-            get_new_task_assignment(self.workers[5],
-                                    Task.Status.AWAITING_PROCESSING)
-
-        # Worker should not be served machine tasks
-        workflow_version = self.workflow_versions['test_workflow_2']
-        simple_machine = self.workflow_steps[
-            workflow_version.slug]['simple_machine']
-        project = Project.objects.create(workflow_version=workflow_version,
-                                         short_description='',
-                                         priority=0,
-                                         task_class=0)
-        Task.objects.create(project=project,
-                            status=Task.Status.AWAITING_PROCESSING,
-                            step=simple_machine)
-
-        with self.assertRaises(NoTaskAvailable):
-            get_new_task_assignment(self.workers[5],
-                                    Task.Status.AWAITING_PROCESSING)
-
-    def test_get_new_task_assignment_reviewer(self):
-        # Reviewer assignment
-        self.assertEquals(Task.objects
-                          .filter(status=Task.Status.PENDING_REVIEW)
-                          .count(),
-                          1)
-
-        # assign a review task to worker
-        assignment = get_new_task_assignment(self.workers[7],
-                                             Task.Status.PENDING_REVIEW)
-        self.assertTrue(assignment is not None)
-        self.assertEquals(assignment.task.status,
-                          Task.Status.REVIEWING)
-
-        self.assertEquals(assignment.in_progress_task_data,
-                          {'test_key': 'test_value'})
-
-        # No tasks in state PENDING_REVIEW
-        # No more tasks left in AWAITING_PROCESSING
-        with self.assertRaises(NoTaskAvailable):
-            get_new_task_assignment(self.workers[7],
-                                    Task.Status.PENDING_REVIEW)
-
-        # Assign an entry-level task to reviewer
-        assignment = get_new_task_assignment(self.workers[7],
-                                             Task.Status.AWAITING_PROCESSING)
-        with self.assertRaises(NoTaskAvailable):
-            get_new_task_assignment(self.workers[7],
-                                    Task.Status.AWAITING_PROCESSING)
+                assert_new_task_status_valid(status)
 
     def test_is_worker_assigned(self):
         task = self.tasks['review_task']
@@ -157,9 +121,6 @@ class BasicTaskLifeCycleTestCase(OrchestraTestCase):
             task__status=Task.Status.POST_REVIEW_PROCESSING)
         self.assertTrue(assignments.exists())
         self.assertTrue(worker_assigned_to_rejected_task(self.workers[4]))
-        with self.assertRaises(TaskAssignmentError):
-            get_new_task_assignment(self.workers[4],
-                                    Task.Status.AWAITING_PROCESSING)
 
     def test_worker_has_reviewer_status(self):
         self.assertFalse(worker_has_reviewer_status(self.workers[0]))
@@ -178,11 +139,11 @@ class BasicTaskLifeCycleTestCase(OrchestraTestCase):
 
         # Create first task in test project
         create_subsequent_tasks(project)
-        self.assertEquals(project.tasks.count(), 1)
+        self.assertEqual(project.tasks.count(), 1)
         # Assign initial task to worker 0
         task = project.tasks.first()
         counter = role_counter_required_for_new_task(task)
-        self.assertEquals(counter, 0)
+        self.assertEqual(counter, 0)
 
         initial_task = assign_task(self.workers[0].id,
                                    task.id)
@@ -194,7 +155,7 @@ class BasicTaskLifeCycleTestCase(OrchestraTestCase):
                                        self.workers[0])
 
             counter = role_counter_required_for_new_task(initial_task)
-            self.assertEquals(counter, 1)
+            self.assertEqual(counter, 1)
 
             initial_task = assign_task(self.workers[1].id,
                                        task.id)
@@ -202,7 +163,7 @@ class BasicTaskLifeCycleTestCase(OrchestraTestCase):
                                        Iteration.Status.REQUESTED_REVIEW,
                                        self.workers[1])
             counter = role_counter_required_for_new_task(initial_task)
-            self.assertEquals(counter, 2)
+            self.assertEqual(counter, 2)
 
     def test_assign_task(self):
         entry_task = TaskFactory(
@@ -268,22 +229,22 @@ class BasicTaskLifeCycleTestCase(OrchestraTestCase):
             current_assignment(review_task).in_progress_task_data, test_data)
 
         # Assign review task to review worker
-        self.assertEquals(review_task.assignments.count(), 1)
+        self.assertEqual(review_task.assignments.count(), 1)
         review_task = assign_task(self.workers[3].id, review_task.id)
-        self.assertEquals(review_task.assignments.count(), 2)
+        self.assertEqual(review_task.assignments.count(), 2)
 
         reviewer_assignment = current_assignment(review_task)
         self.assertEqual(
             reviewer_assignment.worker, self.workers[3])
         self.assertEqual(
             reviewer_assignment.in_progress_task_data, test_data)
-        self.assertEquals(
+        self.assertEqual(
             reviewer_assignment.iterations.count(), 1)
         self.assertEqual(
             reviewer_assignment.iterations.first().start_datetime,
             reviewer_assignment.start_datetime)
 
-        self.assertEquals(
+        self.assertEqual(
             review_task.status, Task.Status.REVIEWING)
 
     def test_get_task_overview_for_worker(self):
@@ -297,7 +258,8 @@ class BasicTaskLifeCycleTestCase(OrchestraTestCase):
             'project': {'details': task.project.short_description,
                         'id': task.project.id,
                         'project_data': {},
-                        'team_messages_url': None},
+                        'status': 'Active',
+                        'scratchpad_url': None},
             'workflow': {'slug': 'w1',
                          'name': 'Workflow One'},
             'workflow_version': {'slug': 'test_workflow',
@@ -311,13 +273,22 @@ class BasicTaskLifeCycleTestCase(OrchestraTestCase):
             'assignment_id': task.assignments.get(worker=self.workers[0]).id,
             'is_reviewer': False,
             'is_read_only': True,
+            'is_project_admin': False,
             'worker': {
                 'username': self.workers[0].user.username,
                 'first_name': self.workers[0].user.first_name,
                 'last_name': self.workers[0].user.last_name,
             }
         }
-        self.assertEquals(data, expected)
+        self.assertEqual(data, expected)
+
+        # Superuser should have access to latest task data
+        superuser = get_user_model().objects.create_superuser(
+            'superuser', 'superuser@b12.io', 'test-password')
+        superworker = Worker.objects.create(user=superuser)
+        data = get_task_overview_for_worker(task.id, superworker)
+        expected['is_project_admin'] = True
+        self.assertEqual(data, expected)
 
     def test_task_assignment_saving(self):
         """
@@ -414,7 +385,7 @@ class BasicTaskLifeCycleTestCase(OrchestraTestCase):
                               'rate': 1,
                               'max_reviews': 1}
         step.save()
-        self.assertEquals(
+        self.assertEqual(
             get_next_task_status(task,
                                  Iteration.Status.REQUESTED_REVIEW),
             Task.Status.PENDING_REVIEW)
@@ -423,13 +394,13 @@ class BasicTaskLifeCycleTestCase(OrchestraTestCase):
                               'rate': 0,
                               'max_reviews': 1}
         step.save()
-        self.assertEquals(
+        self.assertEqual(
             get_next_task_status(task,
                                  Iteration.Status.REQUESTED_REVIEW),
             Task.Status.COMPLETE)
 
         task.status = Task.Status.POST_REVIEW_PROCESSING
-        self.assertEquals(
+        self.assertEqual(
             get_next_task_status(task,
                                  Iteration.Status.REQUESTED_REVIEW),
             Task.Status.REVIEWING)
@@ -441,7 +412,7 @@ class BasicTaskLifeCycleTestCase(OrchestraTestCase):
                               'rate': 1,
                               'max_reviews': 0}
         step.save()
-        self.assertEquals(
+        self.assertEqual(
             get_next_task_status(task,
                                  Iteration.Status.REQUESTED_REVIEW),
             Task.Status.COMPLETE)
@@ -450,7 +421,7 @@ class BasicTaskLifeCycleTestCase(OrchestraTestCase):
                               'rate': 1,
                               'max_reviews': 2}
         step.save()
-        self.assertEquals(
+        self.assertEqual(
             get_next_task_status(task,
                                  Iteration.Status.REQUESTED_REVIEW),
             Task.Status.PENDING_REVIEW)
@@ -467,7 +438,7 @@ class BasicTaskLifeCycleTestCase(OrchestraTestCase):
                               'rate': 1,
                               'max_reviews': 1}
         step.save()
-        self.assertEquals(
+        self.assertEqual(
             get_next_task_status(task,
                                  Iteration.Status.REQUESTED_REVIEW),
             Task.Status.COMPLETE)
@@ -477,7 +448,7 @@ class BasicTaskLifeCycleTestCase(OrchestraTestCase):
 
         # Create first task in test project
         create_subsequent_tasks(project)
-        self.assertEquals(project.tasks.count(), 1)
+        self.assertEqual(project.tasks.count(), 1)
         # Assign initial task to worker 0
         initial_task = assign_task(self.workers[0].id,
                                    project.tasks.first().id)
@@ -487,19 +458,19 @@ class BasicTaskLifeCycleTestCase(OrchestraTestCase):
             initial_task = submit_task(initial_task.id, {},
                                        Iteration.Status.REQUESTED_REVIEW,
                                        self.workers[0])
-        self.assertEquals(project.tasks.count(), 2)
+        self.assertEqual(project.tasks.count(), 2)
         related_task = project.tasks.exclude(id=initial_task.id).first()
         # Worker 0 not certified for related tasks, so should not have been
         # auto-assigned
-        self.assertEquals(related_task.assignments.count(), 0)
-        self.assertEquals(related_task.status, Task.Status.AWAITING_PROCESSING)
+        self.assertEqual(related_task.assignments.count(), 0)
+        self.assertEqual(related_task.status, Task.Status.AWAITING_PROCESSING)
 
         # Reset project
         project.tasks.all().delete()
 
         # Create first task in test project
         create_subsequent_tasks(project)
-        self.assertEquals(project.tasks.count(), 1)
+        self.assertEqual(project.tasks.count(), 1)
         # Assign initial task to worker 0
         initial_task = assign_task(self.workers[0].id,
                                    project.tasks.first().id)
@@ -518,7 +489,7 @@ class BasicTaskLifeCycleTestCase(OrchestraTestCase):
 
         # Create first task in test project
         create_subsequent_tasks(project)
-        self.assertEquals(project.tasks.count(), 1)
+        self.assertEqual(project.tasks.count(), 1)
         # Assign initial task to worker 4
         initial_task = assign_task(self.workers[4].id,
                                    project.tasks.first().id)
@@ -528,16 +499,23 @@ class BasicTaskLifeCycleTestCase(OrchestraTestCase):
             initial_task = submit_task(initial_task.id, {},
                                        Iteration.Status.REQUESTED_REVIEW,
                                        self.workers[4])
-        self.assertEquals(project.tasks.count(), 2)
+        self.assertEqual(project.tasks.count(), 2)
         related_task = project.tasks.exclude(id=initial_task.id).first()
         # Worker 4 is certified for related task and should have been assigned
-        self.assertEquals(related_task.assignments.count(), 1)
-        self.assertEquals(related_task.status, Task.Status.PROCESSING)
+        self.assertEqual(related_task.assignments.count(), 1)
+        self.assertEqual(related_task.status, Task.Status.PROCESSING)
         self.assertTrue(
             related_task.is_worker_assigned(self.workers[4]))
 
-        # Reset project
-        project.tasks.all().delete()
+    def test_todolist_templates_to_apply(self):
+        project = self.projects['assignment_policy']
+        mock = MagicMock(return_value=True)
+        with patch('orchestra.utils.task_lifecycle.add_todolist_template',
+                   new=mock):
+            # Create first task in test project
+            create_subsequent_tasks(project)
+            assert mock.called_once
+            assert mock.call_args[0][0] == 'project-checklist'
 
     def test_malformed_assignment_policy(self):
         project = self.projects['assignment_policy']
@@ -560,12 +538,13 @@ class BasicTaskLifeCycleTestCase(OrchestraTestCase):
                     }
                 }
             },
+            creation_policy=get_default_creation_policy(),
         )
         malformed_step.creation_depends_on.add(first_step)
 
         # Create first task in project
         create_subsequent_tasks(project)
-        self.assertEquals(project.tasks.count(), 1)
+        self.assertEqual(project.tasks.count(), 1)
 
         # Assign initial task to worker 0 and mark as complete
         initial_task = assign_task(self.workers[4].id,
@@ -596,7 +575,168 @@ class BasicTaskLifeCycleTestCase(OrchestraTestCase):
         with self.assertRaises(AssignmentPolicyError):
             create_subsequent_tasks(project)
 
-        # Reset workflow and project
-        first_step.assignment_policy = {}
-        first_step.save()
-        project.tasks.all().delete()
+    def test_malformed_creation_policy(self):
+        project = self.projects['creation_policy']
+        workflow_version = project.workflow_version
+        first_step = self.workflow_steps[
+            workflow_version.slug]['creation_policy_step_0']
+
+        # Create an invalid machine step with an assignment policy
+        malformed_step = StepFactory(
+            workflow_version=workflow_version,
+            slug='machine_step',
+            is_human=False,
+            creation_policy={},
+        )
+        malformed_step.creation_depends_on.add(first_step)
+
+        # Create first task in project
+        create_subsequent_tasks(project)
+        self.assertEqual(project.tasks.count(), 1)
+
+        # Assign initial task to worker 0 and mark as complete
+        initial_task = assign_task(self.workers[4].id,
+                                   project.tasks.first().id)
+        initial_task.status = Task.Status.COMPLETE
+        initial_task.save()
+
+        # Cannot have an invalid blob for the creation_policy
+        with self.assertRaises(CreationPolicyError):
+            create_subsequent_tasks(project)
+
+    @patch('orchestra.utils.task_lifecycle.schedule_machine_tasks')
+    def test_schedule_machine_tasks(self, mock_schedule):
+        project = self.projects['test_human_and_machine']
+
+        # Create first task in project
+        create_subsequent_tasks(project)
+
+        # Assign initial task to worker 0 and mark as complete
+        initial_task = assign_task(self.workers[0].id,
+                                   project.tasks.first().id)
+        initial_task.status = Task.Status.COMPLETE
+        initial_task.save()
+
+        create_subsequent_tasks(project)
+        self.assertEqual(mock_schedule.call_count, 1)
+        self.assertEqual(mock_schedule.call_args[0][0], project)
+        steps = list(project.workflow_version.steps.filter(is_human=False))
+        self.assertEqual(mock_schedule.call_args[0][1], steps)
+
+    @patch('orchestra.utils.task_lifecycle._preassign_workers')
+    @patch('orchestra.utils.task_lifecycle.schedule_machine_tasks')
+    def test_schedule_machine_tasks_failed(self, mock_schedule,
+                                           mock_preassign):
+        project = self.projects['test_human_and_machine']
+
+        # Create first task in project
+        create_subsequent_tasks(project)
+
+        # Assign initial task to worker 0 and mark as complete
+        initial_task = assign_task(self.workers[0].id,
+                                   project.tasks.first().id)
+        initial_task.status = Task.Status.COMPLETE
+        initial_task.save()
+
+        mock_preassign.side_effect = Exception
+        with self.assertRaises(Exception):
+            create_subsequent_tasks(project)
+        mock_schedule.assert_not_called()
+
+    def test_next_todo_with_earlier_due_time(self):
+        task = self.tasks['next_todo_task']
+        task.step = self.step
+        task.project = self.project
+        task.save()
+        # create todos with different due date times and one without
+        TodoFactory(
+            step=self.step,
+            project=self.project,
+            title='todo1',
+            due_datetime=parse(DEADLINE2_DATETIME))
+
+        TodoFactory(
+            step=self.step,
+            project=self.project,
+            title='todo2')
+
+        tasks_assigned = tasks_assigned_to_worker(self.workers[5])
+
+        for t in tasks_assigned:
+            if t['id'] == task.id:
+                next_todo_due = t['next_todo_dict'].get('due_datetime', None)
+
+                # should select the todo with a deadline
+                self.assertEqual(next_todo_due, DEADLINE2_DATETIME)
+                self.assertEqual(t['should_be_active'], True)
+
+        TodoFactory(
+            step=self.step,
+            project=self.project,
+            title='todo3',
+            due_datetime=parse(DEADLINE1_DATETIME))
+
+        tasks_assigned = tasks_assigned_to_worker(self.workers[5])
+        for t in tasks_assigned:
+            if t['id'] == task.id:
+                next_todo_due = t['next_todo_dict'].get('due_datetime', None)
+
+                # should select the todo with earliest deadline
+                self.assertEqual(next_todo_due, DEADLINE1_DATETIME)
+                self.assertEqual(t['should_be_active'], True)
+
+    @patch('orchestra.utils.task_lifecycle.timezone')
+    def test_next_todo_with_earlier_start_time(self, mock_timezone):
+        mock_timezone.now = MagicMock(return_value=parse(
+                                        MOCK_CURRENT))
+        task = self.tasks['next_todo_task']
+        task.step = self.step
+        task.project = self.project
+        task.save()
+        # create todos with different due date times and one without
+        TodoFactory(
+            step=self.step,
+            project=self.project,
+            title='todo1',
+            start_by_datetime=parse(DEADLINE2_DATETIME))
+
+        TodoFactory(
+            step=self.step,
+            project=self.project,
+            title='todo2',
+            start_by_datetime=parse(DEADLINE1_DATETIME))
+
+        tasks_assigned = tasks_assigned_to_worker(self.workers[5])
+        for t in tasks_assigned:
+            if t['id'] == task.id:
+                next_todo_start = t['next_todo_dict'].get(
+                    'start_by_datetime', None)
+                # should select the todo with a deadline
+                self.assertEqual(next_todo_start, DEADLINE1_DATETIME)
+                self.assertEqual(t['should_be_active'], False)
+
+
+class EndProjectTestCase(OrchestraTransactionTestCase):
+    def setUp(self):
+        super().setUp()
+        setup_models(self)
+
+    @patch('orchestra.utils.task_lifecycle.archive_project_slack_group')
+    @patch('orchestra.tests.helpers.workflow.abort_cleanup_function')
+    def test_end_project_calls_abort_completion_function(
+            self, mock_abort_cleanup_fn, mock_archive_project_slack_group):
+        path = 'orchestra.tests.helpers.workflow.abort_cleanup_function'
+        abort_completion_function = {
+            'path': path,
+            'kwargs': {
+                'some_key': 'some_value'
+            }
+        }
+        project = self.projects['test_human_and_machine']
+        project.workflow_version.abort_completion_function = (
+            abort_completion_function)
+        project.workflow_version.save()
+        end_project(project.id)
+        kwargs = abort_completion_function['kwargs']
+        mock_abort_cleanup_fn.assert_called_with(
+            project, **kwargs)
